#include "Engine.h"
#include "SDL_KeyMap.h"
float maxAnisotropy=0;
bool gAnistropy = false;
bool gVsync = false;
bool gAntiAliasing = true;
int gAASamples = 4;

Engine* gpEng = NULL;
/**************************************************************************/
/* initialize engine													  */
/**************************************************************************/
Engine::Engine(int w, int h, bool fscreen)
{
	gpEng = this;
	gLog.SetLogState(1);

	if(!UTIL_SDL::InitSDL("LD24 X-0ut",w,h,32,fscreen,gAntiAliasing?gAASamples:0))
		return;
	glClear(GL_COLOR_BUFFER_BIT);
	UTIL_SDL::InitJoystick();
	LoadInput();
	SDL_GL_SwapBuffers();
	winWidth = w;
	winHeight = h;
	AspectRatio = RATIO_16_9;
	bTextInput = false;
	bCfgJoystick = false;
	mbPaused = false;
	mbQuit = false;
	UTIL_Misc::Seed(1000);
	mbUseEditor = true;

	SDL_WM_GrabInput(SDL_GRAB_ON);

	InitDevil();

//	if(!InitExtensions())
//		return;
//	if(!InitShaders())
//		return;
//	if(!InitializeFBO(mFBO,"MAIN", winWidth, winHeight, true, false, true, true, false, GL_TEXTURE_RECTANGLE_ARB, 2))
//		return;

	if(!mFont1.LoadFont("Fonts.psd"))
		return;
	if(!mFont2.LoadFont("Fonts2.psd"))
		return;
	mFont2.SetShadowImage(mFont1.texID);
	if(!mFont3.LoadFont("Fonts3.psd"))
		return;

	if(!(mSprImg = UTIL_GL::Image::LoadImage("Data/Gfx/SpriteSheet.png", GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return;
	mSpr.Split(mSprImg,8,8,64,64);
	mSpr2.Split(mSprImg,4,4,64,64);

	if(!InitializeAudio(128))
		return;

	if(!LoadMusic("Data/Music/8Bit-LD.mp3",mMusic))
		return;

	InitializePhysics();
	mScroll = Vec3(500,0,0);
	mScrollVel = Vec3(20.5f,0,0);
	miSegments = 0;
	miTakeDamageTime = 0;
	miScore = 0;
	mbCoreDestroyed = false;
	miScreenFlashTime = 0;
	miInternalClock = -200;
	miPlayerDeathTime = 0;
	miHomeRun = 0;

	gLog.OutPut("adding some dynamic rigid bodies\n");
/*	btCollisionShape* boxShape = new btBoxShape(btVector3(5,5,5));
	btVector3 boxinertia;
	boxShape->calculateLocalInertia(1.5f,boxinertia);
	Vec3 boxinert(boxinertia.x(),boxinertia.y(),boxinertia.z());
	for(int n=0; n<15; n++)
	{
		InsertNewRigidBody(boxShape,Vec3(UTIL_Misc::Rand()%360,UTIL_Misc::Rand()%360,UTIL_Misc::Rand()%360),
			Vec3(150,150 + n*50,500),
		1.0f, boxinert,1);
	}
*/
	mTimer.Reset();
	gLog.OutPut("READY!\n");
	while(mTimer.GetTime()<1)
	{
		mTimer.Update();
	}
	mTimer.Reset();
	PlayMusic(mMusic,true,0);
	Pump();

}
void Engine::ResetGame()
{
	printf("reset game\n");
	if(!mBulletRigidBodies.empty())
	{
		std::list<btRigidBody*>::iterator IT;
		for(IT=mBulletRigidBodies.begin(); IT!=mBulletRigidBodies.end(); IT++)
		{
			mBulletDynamicsWorld->removeRigidBody((*IT));
			delete (*IT)->getMotionState();
			delete (*IT);
		}
		mBulletRigidBodies.clear();
	}
	if(!mGameFX.empty())
	{
		std::list<Effect*>::iterator FX;
		for(FX=mGameFX.begin(); FX!=mGameFX.end(); FX++)
		{
			delete (*FX);
		}
		mGameFX.clear();
	}
	if(!mGameEnts.empty())
	{
		std::list<Entity*>::iterator E;
		for(E=mGameEnts.begin(); E!=mGameEnts.end(); E++)
		{
			delete (*E);
		}
		mGameEnts.clear();
	}

	printf("init scene\n");
	InitScene();
	mScroll = Vec3(500,0,0);
	mScrollVel = Vec3(20.5f,0,0);
	miSegments = 0;
	miTakeDamageTime = 0;
	miScore = 0;
	mbCoreDestroyed = false;
	miScreenFlashTime = 0;
	miInternalClock = -200;
	miPlayerDeathTime = 0;
	miHomeRun = 0;

//	printf("play music\n");
	PlayMusic(mMusic,true,0);

}
/**************************************************************************/
/* initialize image library												  */
/**************************************************************************/
void Engine::InitDevil()
{
	gLog.OutPut("\n[Initializing Devil]\n");
	ilInit();
	iluInit();
	ilutInit();
	gLog.OutPut("Complete...\n\n");
}
/**************************************************************************/
/* engine is shutting down												  */
/**************************************************************************/
Engine::~Engine()
{
	ShutDownPhysics();
	ShutDownAudio();
}

/**************************************************************************/
/* engine pump, everything is called from here							  */
/**************************************************************************/
void Engine::Pump()
{
	mMouseMotion = Vec2(0,0);
	mTimer.Reset();
	char FPS[32];
	accumulator=0;
	SDL_Event event;
	while(!mbQuit)
	{
		mTimer.Update();
		accumulator += mTimer.GetDelta();
		while(accumulator>=TIME_STEP)
		{
			while(SDL_PollEvent(&event)>0)
			{

				switch(event.type)
				{
					case SDL_KEYDOWN:
						if(bTextInput)
							TextInput(event.key.keysym);
						else
						KeyStateAlltered(event.key.keysym.sym,true);
					break;
					case SDL_KEYUP:
					KeyStateAlltered(event.key.keysym.sym,false);
					break;
					//joystick buttons
					case SDL_JOYBUTTONDOWN:
						{
						//	if(!bJoystickSupported)
						//		break;
							int b = event.jbutton.button;
							if(b<20)
								KeyStateAlltered(JOYSTICK_BUTTON+b, true);				
						}
						break;
					case SDL_JOYBUTTONUP:
						{
						//	if(!bJoystickSupported)
						//		break;
							int b = event.jbutton.button;
							if(b<20)
								KeyStateAlltered(JOYSTICK_BUTTON+b, false);				
						}
						break;//*/
					//joystick motion
					case SDL_JOYAXISMOTION:
					//	if(!bJoystickSupported)
					//		break;
						CheckJoystickMotion(event);
						break;//*/

					case SDL_QUIT:
						mbQuit = true;
					break;

					case SDL_MOUSEBUTTONDOWN:
					KeyStateAlltered(event.button.button,true);
					break;
					case SDL_MOUSEBUTTONUP:
					KeyStateAlltered(event.button.button,false);
					break;

					case SDL_MOUSEMOTION:
						mMouseMotion.x = (((float)event.motion.xrel * mfMouseSensi));
						mMouseMotion.y = (((float)event.motion.yrel * mfMouseSensi));
						break;
				}
			}
			SDL_GetMouseState(&MouseX,&MouseY);
			Update();
			mMouseMotion = Vec2(0,0);
			if(!mbPaused)
				miInternalClock++;

			KeysPostUpdate();
			accumulator-=TIME_STEP;
		}

		const float interp = mInterp = accumulator/TIME_STEP;
		if(mbPaused)
			Render(1);
		else
			Render(interp);
			UTIL_GL::GetGLError();

		//	UTIL_GL::GL2D::SetOrtho(640,360);

		SDL_GL_SwapBuffers();
		SDL_Delay(1);
	}
}


/**************************************************************************/
/* update logic															  */
/**************************************************************************/
	static float R = 45, oldR = 45;
	static float frame = 0;
void Engine::Update()
{
	if(KEY_QUIT.state == JUST_PRESSED && !mbPaused)
	{
		mbQuit =true;
	}
	if(KEY_PAUSE.state == JUST_PRESSED)
	{
		mbPaused = !mbPaused;
		mbPaused?
			PauseAudio():UnPauseAudio();
	}
	if(mbPaused)
		return;

	mFontW.remove_if(PruneFontWriters);

	if(miPlayerDeathTime<=0)
	{
	if(miInternalClock == -190)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "I OPEN MY EYES AND SEE THE PATH BEFORE ME.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 0)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "WASD TO CONTROL ME, MOUSE TO ARM ME.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 400)
	{
		btVector3 boxinertia;
		float C = UTIL_Misc::RandFloat(0.5f);
		Entity* Ent;
		btRigidBody* RB;
		gpEng->mBoxShape->calculateLocalInertia(5.5f,boxinertia);
		Vec3 boxinert(boxinertia.x(),boxinertia.y(),boxinertia.z());
		Ent = new Entity( EntityShape(Vec3(8,8,8), Vec3(1,0,0)), ENT_MEDBOX);
		Ent->mEntShapes.push_back(EntityShape(Vec3(-6,-6,-6), Vec3(4,4,4), Vec3(0.2f,0.2f,0.2f)));
		Ent->mEntShapes.push_back(EntityShape(Vec3(6,-6,-6), Vec3(4,4,4), Vec3(0.2f,0.2f,0.2f)));
		Ent->mEntShapes.push_back(EntityShape(Vec3(-6,-6,6), Vec3(4,4,4), Vec3(0.2f,0.2f,0.2f)));
		Ent->mEntShapes.push_back(EntityShape(Vec3(6,-6,6), Vec3(4,4,4), Vec3(0.2f,0.2f,0.2f)));
		Ent->mEntShapes.push_back(EntityShape(Vec3(-6,6,-6), Vec3(4,4,4), Vec3(0.2f,0.2f,0.2f)));
		Ent->mEntShapes.push_back(EntityShape(Vec3(6,6,-6), Vec3(4,4,4), Vec3(0.2f,0.2f,0.2f)));
		Ent->mEntShapes.push_back(EntityShape(Vec3(-6,6,6), Vec3(4,4,4), Vec3(0.2f,0.2f,0.2f)));
		Ent->mEntShapes.push_back(EntityShape(Vec3(6,6,6), Vec3(4,4,4), Vec3(0.2f,0.2f,0.2f)));
		Ent->miLife = 60;
		RB = gpEng->InsertNewRigidBody(gpEng->mBoxShapeMed,Vec3(UTIL_Misc::Rand()%360,UTIL_Misc::Rand()%360,UTIL_Misc::Rand()%360),
		Vec3(-mScroll.x+500,180,-70),
		10.0f, boxinert, Ent, COL_ENTITY, COL_ENTITY | COL_LASER | COL_PLR | COL_STATIC);
		RB->applyCentralImpulse(btVector3(-50,0,0));
		gpEng->mGameEnts.push_back(Ent);
	}
	else if(miInternalClock == 500)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "DESTROY COMPANION CUBES TO EVOLVE ME.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 800)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "DONT LET ME DE-EVOLVE OR THE MISSION WILL BE LOST.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 3000)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "AS I BATTLED ONWARD IT BECAME OBVIOUS", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 3300)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "I NEEDED TO EVOLVE TO SURVIVE THIS NIGHTMARE", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 3600)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "BUT THE STRONGER I BECAME", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 3900)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "THE DEEPER THE SEA OF DESPAIR ENGULFED ME.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 6000)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "THE ENEMY STRONGHOLD WAS FORMIDABLE.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 6300)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "I PONDERED HOW MANY OF MY COMPANIONS", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 6600)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "HAD FAILED BEFORE ME.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 6900)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "BUT I MUST PUSH FOWARDS,", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 7200)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "I'VE COME TOO FAR TO LOOK BACK NOW.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 10000)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "JUST LIKE THE COGS IN THIS GREAT MACHINE", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 10200)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "THE ONSLAUGHT NEVER WANED.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 10400)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "RELENTLESS.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 14900)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "SO THIS IS WHAT HAS PLAGUED US FOR SO LONG.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 15100)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "MY CONSCIOUS WAS NOW CLEAR", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 15300)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "AND MY ACTIONS IRREFUTABLE.", 2,160);
		mFontW.push_back(FW);
	}
	else if(miInternalClock == 15500)
	{
		FontWriter FW; mFontW.clear();
		FW.Spawn(Vec2(160,170), "DESTROY THE CORE!", 2,160);
		mFontW.push_back(FW);
	}
	}

	if(miInternalClock<0)
	{
		((Entity*)mPlrBody->getUserPointer())->Update();
		return;
	}
	mScroll -= mScrollVel*0.02f;

	if(miPlayerDeathTime>0)
	{
		miPlayerDeathTime--;
		if(miPlayerDeathTime==1)
		{
			ResetGame();
			return;
		}
	}
	if(miTakeDamageTime>0)
		miTakeDamageTime--;
	if(miScreenFlashTime>0)
		miScreenFlashTime--;

	if(mbCoreDestroyed)
	{
		mVibration = Vec2((UTIL_Misc::Rand()%10)-5,(UTIL_Misc::Rand()%10)-5);
		if(miHomeRun <= 4700)
			mScrollVel.x -= 0.1f;
		if(mScrollVel.x<-70)
			mScrollVel.x=-70;
		miHomeRun++;

		if(miHomeRun == 200)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "NO MATTER WHAT HAPPENS NOW", 2,160);
			mFontW.push_back(FW);
		}
		else if(miHomeRun == 400)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "THE MISSION IS A SUCCESS.", 2,160);
			mFontW.push_back(FW);
		}
		else if(miHomeRun == 600)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "MY BROTHERS LIVES HAVE NOT BEEN SQUANDERED.", 2,160);
			mFontW.push_back(FW);
		}
		else if(miHomeRun == 1000)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "JUST MAYBE I CAN SURVIVE", 2,160);
			mFontW.push_back(FW);
		}
		else if(miHomeRun == 1200)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "IF ONLY I CAN ESCAPE.", 2,160);
			mFontW.push_back(FW);
		}
		else if(miHomeRun == 1800)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "MY ANNIHILATION HAS BEEN TOTAL.", 2,160);
			mFontW.push_back(FW);
		}
		else if(miHomeRun == 2000)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "THIS ALIEN ARCHITECTURE CRUMBLES BENEATH ME.", 2,160);
			mFontW.push_back(FW);
		}

		else if(miHomeRun == 4700)
		{
			gpEng->miScreenFlashTime = 75;
			if(!gpEng->mGameEnts.empty())
			{
				std::list<Entity*>::iterator E;
				for(E=gpEng->mGameEnts.begin(); E!=gpEng->mGameEnts.end(); E++)
				{
					if((*E)->mType == ENT_BOXGEN)
					{
						((BoxGenerator*)(*E))->mbInfiniteBoxes = false;
						((BoxGenerator*)(*E))->miMaxBoxes = 0;
						(*E)->mRemove = true;
					}
					else if((*E)->mType == ENT_BOX || (*E)->mType == ENT_MEDBOX)
					{
						(*E)->mbTagged = true;
					}
				}
			}			
		}
		else if(miHomeRun == 4800)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "I DID IT!", 2,160);
			mFontW.push_back(FW);
		}
		else if(miHomeRun == 5000)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "I SURVIVED!", 2,160);
			mFontW.push_back(FW);
		}
		else if(miHomeRun == 5200)
		{
			FontWriter FW; mFontW.clear();
			FW.Spawn(Vec2(160,170), "THANKYOU.", 2,160);
			mFontW.push_back(FW);
		}

		if(!mBulletRigidBodies.empty())
		{
			std::list<btRigidBody*>::iterator IT;
			for(IT=mBulletRigidBodies.begin(); IT!=mBulletRigidBodies.end(); IT++)
			{
				if(((Entity*)(*IT)->getUserPointer())->mType == ENT_STATIC)
				{
					btTransform trans;
					(*IT)->getMotionState()->getWorldTransform(trans);
					btVector3 btP = trans.getOrigin();
					if(btP.x()>-mScroll.x-300)
					{
						(*IT)->setLinearFactor(btVector3(1,1,0));
						(*IT)->setAngularFactor(btVector3(0,0,1));
						(*IT)->setGravity(btVector3(0,0,0));
						(*IT)->activate();						
					}
				}
			}
		}//*/
	}

	if(!mbCoreDestroyed)
	if(mScroll.x<-5750)
	{
		mScroll.x = -5750;
		if(mScrollVel.x>0)
		{
			BoxGenerator* mGen2 = new BoxGenerator(EntityShape(Vec3(1,1,1),Vec3(1,1,1)),ENT_BOXGEN);
			mGen2->mForce = Vec3(-150,0,0);
			mGen2->mPos = Vec3(-mScroll.x+500,180,-70);
			mGen2->miRadius = 100;
			mGen2->miMaxBoxes = 100;
			mGen2->miBoxFrequency = 1;
			mGen2->mbInfiniteBoxes = true;
			mGameEnts.push_back(mGen2);
		}
		mScrollVel.x = 0;
	}
	//start big boxes
	if(miInternalClock == 12600)
	{
		BoxGenerator* mGen2 = new BoxGenerator(EntityShape(Vec3(1,1,1),Vec3(1,1,1)),ENT_BOXGEN);
		mGen2->mForce = Vec3(-10,30,0);
		mGen2->mPos = Vec3(-mScroll.x+500,180,-70);
		mGen2->miRadius = 100;
		mGen2->miMaxBoxes = 100;
		mGen2->miBoxFrequency = 15;
		mGen2->mbInfiniteBoxes = false;
		mGen2->bBigBoxes = true;
		mGameEnts.push_back(mGen2);
	}

	if(miPlayerDeathTime<=0)
	{
		Vec3 PlrVel(0,0,0);
	if(KEY_LEFT.state == PRESSED)
		PlrVel.x = -200;
	if(KEY_RIGHT.state == PRESSED)
		PlrVel.x = 200; 
	if(KEY_UP.state == PRESSED)
		PlrVel.y = 200;
	if(KEY_DOWN.state == PRESSED)
		PlrVel.y = -200;
	{
		btTransform trans;
		mPlrBody->getMotionState()->getWorldTransform(trans);
		btVector3 btP = trans.getOrigin();
		Vec2 PlrPos(btP.x(), btP.y());

		for(int n=0; n<miSegments; n++)
		{
			mPlrBodySegs[n]->getMotionState()->getWorldTransform(trans);
			btVector3 btP2 = trans.getOrigin();

			Vec3 Rnd((UTIL_Misc::Rand()%50)-25,(UTIL_Misc::Rand()%50)-25,(UTIL_Misc::Rand()%50)-25);
			Vec3 Dir = ((Vec3(btP.x(), btP.y(), btP.z())+Rnd) - Vec3(btP2.x(), btP2.y(), btP2.z()))  * 3.1f;
			mPlrBodySegs[n]->applyCentralImpulse(btVector3(Dir.x, Dir.y, Dir.z));
		}
		if(PlrPos.x<-mScroll.x-260)
			PlrVel.x += 200;
		if(PlrPos.x>-mScroll.x+320)
			PlrVel.x += -200;
		if(PlrPos.y<-mScroll.y+20)
			PlrVel.y += 200;
		if(PlrPos.y>-mScroll.y+340)
			PlrVel.y += -200;

		if(miPlayerDeathTime<=0)
		if(PlrPos.x<-mScroll.x-270 || PlrPos.x>-mScroll.x+330 || PlrPos.y<-mScroll.y+10 || PlrPos.y>-mScroll.y+350)
		{
			miScreenFlashTime = 75;
			miTakeDamageTime = 200;
			miPlayerDeathTime = 150;
			MasterMusicFadeout(500);
			FontWriter FW;
			mFontW.clear();
			FW.Spawn(Vec2(160,170), "I HAVE WATCHED MY DREAMS SHATTER.", 2,80);
			mFontW.push_back(FW);
		}

	}
	mPlrBody->applyCentralImpulse(btVector3(PlrVel.x,PlrVel.y,PlrVel.z));
	

	if(KEY_ATTACK1.state == PRESSED)
	{
		for(int n=0; n<miSegments+1; n++)
		{
		btCollisionShape* boxShape = new btBoxShape(btVector3(5,1,1));
		btVector3 boxinertia;
		boxShape->calculateLocalInertia(0.5f,boxinertia);
		Vec3 boxinert(boxinertia.x(),boxinertia.y(),boxinertia.z());

		btTransform trans;
		if(n==0)
			mPlrBody->getMotionState()->getWorldTransform(trans);
		else
			mPlrBodySegs[n-1]->getMotionState()->getWorldTransform(trans);

		btVector3 btP = trans.getOrigin();
		Vec2 PlrPos(btP.x(), btP.y());
		Vec2 MP(MouseX-320-mScroll.x,360-MouseY-mScroll.y);
		Vec2 Dir = Normalize(PlrPos-MP)*50;
		float Angle = Dir.Angle(Vec2(1,0));

		Laser* Ent = new Laser( EntityShape(Vec3(5,1,1), Vec3(1,1,1)), ENT_LASER);
		btRigidBody* RB = InsertNewRigidBody(boxShape,Vec3(0,0,Angle),
		Vec3(btP.x(),btP.y(),btP.z()),
		0.1f, boxinert, Ent, COL_LASER, COL_ENTITY | COL_STATIC);
		RB->applyCentralImpulse(btVector3(-Dir.x,-Dir.y,0));
		RB->setGravity(btVector3(0,0,0));
		RB->setAngularFactor(btVector3(0,0,0));
		mGameEnts.push_back(Ent);
		}
	}
	mPlrBody->applyCentralImpulse(btVector3(mScrollVel.x,mScrollVel.y,mScrollVel.z));
	}
	UpdatePhysics();

	mPlrBody->setLinearVelocity(btVector3(0,0,0));
	mPlrBody->activate();
	for(int n=0; n<miSegments; n++)
	{
		mPlrBodySegs[n]->activate();
	}
	int numManifolds = mBulletDynamicsWorld->getDispatcher()->getNumManifolds();
	for (int i=0;i<numManifolds;i++)
	{
		btPersistentManifold* contactManifold =  mBulletDynamicsWorld->getDispatcher()->getManifoldByIndexInternal(i);
		btCollisionObject* obA = static_cast<btCollisionObject*>(contactManifold->getBody0());
		btCollisionObject* obB = static_cast<btCollisionObject*>(contactManifold->getBody1());

		int numContacts = contactManifold->getNumContacts();
		for (int j=0;j<numContacts;j++)
		{
			btManifoldPoint& pt = contactManifold->getContactPoint(j);
			if (pt.getDistance()<0.f)
			{
				if(((Entity*)obA->getUserPointer())->mType == ENT_PLAYER && ((Entity*)obB->getUserPointer())->mType != ENT_STATIC)
				{
					if(miSegments == 0 && miTakeDamageTime<=0)//player die
					{
						miScreenFlashTime = 75;
						miTakeDamageTime = 200;
						miPlayerDeathTime = 150;
						MasterMusicFadeout(500);
						FontWriter FW;
						mFontW.clear();
						FW.Spawn(Vec2(160,170), "I HAVE WATCHED MY DREAMS SHATTER.", 2,80);
						mFontW.push_back(FW);

					}
					if(miSegments>0 && miTakeDamageTime<=0)
					{
						miTakeDamageTime = 200;
						miSegments--;
						((Entity*)mPlrBodySegs[miSegments]->getUserPointer())->mRemove = true;
					}
				}
				if(((Entity*)obA->getUserPointer())->mType == ENT_LASER)
				{
					if(miInternalClock%5==0)
					{
						btTransform trans;
						((btRigidBody*)obA)->getMotionState()->getWorldTransform(trans);
						btVector3 btP = trans.getOrigin();
						InsertGameFX(Vec3(btP.x(),btP.y(),btP.z()),FX_PARTICLE_SPREAD);
					}
					((Entity*)obA->getUserPointer())->mRemove = true;
					((Entity*)obB->getUserPointer())->TakeDamage(1);
					if(((Entity*)obB->getUserPointer())->mRemove)
					{
						btTransform trans;
						((btRigidBody*)obB)->getMotionState()->getWorldTransform(trans);
						btVector3 btP = trans.getOrigin();
						InsertGameFX(Vec3(btP.x(),btP.y(),btP.z()),FX_EXPLOSION_SMALL);
						if(((Entity*)obB->getUserPointer())->mType == ENT_MEDBOX)
							InsertGameFX(Vec3(btP.x(),btP.y(),btP.z()),FX_PARTICLE_POWER);
					}
				}
				if(((Entity*)obB->getUserPointer())->mType == ENT_LASER)
				{
					((Entity*)obB->getUserPointer())->mRemove = true;
					((Entity*)obA->getUserPointer())->TakeDamage(1);
					if(((Entity*)obA->getUserPointer())->mRemove)
					{
						btTransform trans;
						((btRigidBody*)obA)->getMotionState()->getWorldTransform(trans);
						btVector3 btP = trans.getOrigin();
						InsertGameFX(Vec3(btP.x(),btP.y(),btP.z()),FX_EXPLOSION_SMALL);
						if(((Entity*)obA->getUserPointer())->mType == ENT_MEDBOX)
							InsertGameFX(Vec3(btP.x(),btP.y(),btP.z()),FX_PARTICLE_POWER);
					}
					if(miInternalClock%5==0)
					{
					btTransform trans;
					((btRigidBody*)obB)->getMotionState()->getWorldTransform(trans);
					btVector3 btP = trans.getOrigin();
					InsertGameFX(Vec3(btP.x(),btP.y(),btP.z()),FX_PARTICLE_SPREAD);
					}
				}
				if(((Entity*)obA->getUserPointer())->mbJustSpawned)
				{
					if(((Entity*)obA->getUserPointer())->mType == ENT_BOX)
						((Entity*)obA->getUserPointer())->mRemove =true;
				}
				if(((Entity*)obB->getUserPointer())->mbJustSpawned)
				{
					if(((Entity*)obB->getUserPointer())->mType == ENT_BOX)
						((Entity*)obB->getUserPointer())->mRemove =true;
				}

			}
		}
	}
	mBulletRigidBodies.remove_if(PruneRigidBody);
	mGameEnts.remove_if(PruneGameEntity);
	mGameFX.remove_if(PruneFX);
}
bool PruneRigidBody(btRigidBody* body)
{

	//if(((Entity*)body->getUserPointer())->mType == ENT_BOX )
	{
		btTransform trans;
		body->getMotionState()->getWorldTransform(trans);
		btVector3 btP = trans.getOrigin();
		gpEng->mPlrBody->getMotionState()->getWorldTransform(trans);
		btVector3 btP2 = trans.getOrigin();
		if(btP.x() < -gpEng->mScroll.x-360 || btP.x() > -gpEng->mScroll.x+860|| btP.y() < -gpEng->mScroll.y-220 || btP.y() > -gpEng->mScroll.y+540 )
		{
			if(((Entity*)body->getUserPointer())->mType != ENT_STATIC && ((Entity*)body->getUserPointer())->mType != ENT_CORE)
			{
				((Entity*)body->getUserPointer())->mRemove = true;
		//		if(((Entity*)body->getUserPointer())->mType == ENT_TRIGGER)
		//			if(((Trigger*)body->getUserPointer())->pTarget)
		//				((Trigger*)body->getUserPointer())->pTarget->mRemove = true;
				gpEng->mBulletDynamicsWorld->removeRigidBody(body);
				delete body;
				return true;
			}
			if(((Entity*)body->getUserPointer())->mType == ENT_STATIC)
			{
				if(((Entity*)body->getUserPointer())->pOwner)
				{
					((Entity*)body->getUserPointer())->mRemove = true;
					//printf("elevator gone\n");
					gpEng->mBulletDynamicsWorld->removeRigidBody(body);
					delete body;
					return true;
				}
			}

//			printf("box gone too far!\n");
		}
	}

	if(((Entity*)body->getUserPointer())->mRemove)
	{
//		printf("removed a body\n");
		gpEng->mBulletDynamicsWorld->removeRigidBody(body);
		delete body;
		return true;
	}
	return false;

}
bool PruneGameEntity(Entity* ent)
{
	if(ent->mRemove)
	{
		delete ent;
		return true;
	}
	ent->Update();
	ent->mbJustSpawned = false;
	if(ent->mbTagged)
		ent->mRemove = true;
	return false;

}
bool PruneFX(Effect* FX)
{
	if(FX->mRemove)
	{
		delete FX;
		return true;
	}
	FX->Update();
	return false;
}
/**************************************************************************/
/* convert mouse coordinate to 320x180 coordinates						  */
/**************************************************************************/
Vec2 Engine::MapWindowToInternal(int x, int y)
{
	float fx = (float)x / (float)winWidth;
	float fy = (float)y / (float)winHeight;

	return Vec2((int)((fx*mFBO.w)), (int)((fy*mFBO.h)));
}
/**************************************************************************/
/* get row from matrix													  */
/**************************************************************************/
 void getRow (int row, const float m[16], float v[4])
{
        v[0] = m[0 * 4 + row];
        v[1] = m[1 * 4 + row];
        v[2] = m[2 * 4 + row];
        v[3] = m[3 * 4 + row];
}
/**************************************************************************/
/* render																  */
/**************************************************************************/
void Engine::Render(const float interp)
{
	float fogcolor[4] = {9/255.0f,15/255.0f,49/255.0f,0};

	if(this->mbCoreDestroyed)
	{
	fogcolor[0] = UTIL_Misc::TimeRamp(9/255.0f,49/255.0f,300,600,sin((float)miInternalClock*0.05f)*800);
	fogcolor[2] = UTIL_Misc::TimeRamp(49/255.0f,9/255.0f,300,600,sin((float)miInternalClock*0.05f)*800);
	}
	else
	{
	fogcolor[0] = UTIL_Misc::TimeRamp(9/255.0f,49/255.0f,300,600,sin((float)miInternalClock*0.001f)*800);
	fogcolor[2] = UTIL_Misc::TimeRamp(49/255.0f,9/255.0f,300,600,sin((float)miInternalClock*0.001f)*800);
	}
	glClearColor(fogcolor[0],fogcolor[1],fogcolor[2],0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	UTIL_GL::GL3D::SetPerspective(60,1.77,1,1500);
	float Zoom = -30;
	if(miInternalClock<0)
		Zoom += -200-miInternalClock;
	else Zoom = -230;
	glTranslatef(0,-180,Zoom);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	mFrustum.ExtractPlanes();
	glRotatef(5,0,1,0);
	glTranslatef(mScroll.x,mScroll.y,mScroll.z);
	if(this->mbCoreDestroyed)
	{
		glTranslatef(mVibration.x,mVibration.y,0);
	}

	//setup lighting
	float POS[4] = {0,1,0,0};
	float POS2[4] = {0,-0.2,1,0};
	float POS3[4] = {1,0,0,0};
	float DARKAMBIENT[4]={0.25f,0.25f,0.25f,1};
	float WORLD_SPEC[4]={0.25f,0.25f,0.25f,1};
	float WORLD_DIFFUSE[4]={1,1,1,1};
	float EMISSIVE[4]={0.05f,0.05f,0.05f,1};
	float BLACK[4]={0,0,0,1};
	float WHITE[4]={1,1,1,1}; 
	float FILL_DIFFUSE[4]={0.5f,0.5f,0.3f,1};
	float BACK_DIFFUSE[4]={fogcolor[0]*4,fogcolor[1]*4,fogcolor[2]*4,1};

	glColor3f(1,1,1);
	glEnable(GL_LIGHTING);
	glLightfv(GL_LIGHT0,  GL_POSITION, POS2);
	glLightfv(GL_LIGHT0,  GL_AMBIENT, DARKAMBIENT);
	glLightfv(GL_LIGHT0,  GL_DIFFUSE, WORLD_DIFFUSE);
	glLightfv(GL_LIGHT0,  GL_SPECULAR, WHITE);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT1,  GL_POSITION, POS);
	glLightfv(GL_LIGHT1,  GL_AMBIENT, BLACK);
	glLightfv(GL_LIGHT1,  GL_DIFFUSE, FILL_DIFFUSE);
	glLightfv(GL_LIGHT1,  GL_SPECULAR, DARKAMBIENT);
	glEnable(GL_LIGHT1);
	glLightfv(GL_LIGHT2,  GL_POSITION, POS3);
	glLightfv(GL_LIGHT2,  GL_AMBIENT, BLACK);
	glLightfv(GL_LIGHT2,  GL_DIFFUSE, BACK_DIFFUSE);
	glLightfv(GL_LIGHT2,  GL_SPECULAR, DARKAMBIENT);
	glEnable(GL_LIGHT2);//*/
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
//	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_COLOR_MATERIAL);
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 100);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, BLACK);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, WHITE);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, BLACK);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, EMISSIVE);

	//setup fog
	glEnable(GL_FOG);
	glFogi(GL_FOG_MODE, GL_EXP2);
	glFogf(GL_FOG_DENSITY, 0.001f);
	//glFogf(GL_FOG_DENSITY, 0.003f);//full screen fog
	glHint(GL_FOG_HINT, GL_NICEST);
	glFogfv(GL_FOG_COLOR, fogcolor);

	//background
	glBegin(GL_QUADS);
	glColor3f(fogcolor[0],fogcolor[1],fogcolor[2]);
	for(int n=0; n<13; n++)
	{
		RenderCube(Vec3(-1000+ n*500,0,-500), Vec3(50,50,250));
		RenderCube(Vec3(-1000+ n*500,340,-500), Vec3(50,50,250));

		RenderCube(Vec3(-750+ n*500,200,-700), Vec3(50,750,50));

	}
	glEnd();
	glPushMatrix();
	glTranslatef(-650+ 13*500,160,-700);
	glRotatef(miInternalClock*1.5f,0,1,0);
	glRotatef(miInternalClock*2.5f,1,0,0);
	glBegin(GL_QUADS);
	glColor3f(1,sin((float)miInternalClock*0.5f),0);
	RenderCube(Vec3(250,250,250));
	glColor3f(fogcolor[0],fogcolor[1],fogcolor[2]);
	RenderCube(Vec3(200,0,0), Vec3(100,260,260));
	RenderCube(Vec3(-200,-0,0), Vec3(100,260,260));

	RenderCube(Vec3(250,0,0), Vec3(100,100,100));
	RenderCube(Vec3(-250,0,0), Vec3(100,100,100));
	RenderCube(Vec3(0,250,0), Vec3(100,100,100));
	RenderCube(Vec3(0,-250,0), Vec3(100,100,100));
	RenderCube(Vec3(0,0,250), Vec3(100,100,100));
	RenderCube(Vec3(0,0,-250), Vec3(100,100,100));
	glEnd();
	glPopMatrix();
	//ents
	std::list<btRigidBody*>::iterator RB;
	for(RB=mBulletRigidBodies.begin();RB!=mBulletRigidBodies.end(); RB++)
	{
		if(((Entity*)(*RB)->getUserPointer())->mType == ENT_PLAYER)
			if(miPlayerDeathTime>0)
				continue;
		glPushMatrix();
		//glTranslatef(-25,-15,-70);
		btTransform trans;
		(*RB)->getMotionState()->getWorldTransform(trans);
		float MATRIX[16];
		trans.getOpenGLMatrix(MATRIX);
		glMultMatrixf(MATRIX);
		Entity* Ent = (Entity*)(*RB)->getUserPointer();
		if(Ent->miDmgTime>0)
		{
			glDisable(GL_LIGHTING);
			if(Ent->miDmgTime==2)
				glColor3f(1,1,1);
			if(Ent->miDmgTime==1)
				glColor3f(1,0,0);
		}
		else
		{
			glEnable(GL_LIGHTING);
			std::list<EntityShape>::iterator ES;
			for(ES=Ent->mEntShapes.begin(); ES!=Ent->mEntShapes.end(); ES++)
				glColor3fv((*ES).Color.v);
		}
		glBegin(GL_QUADS);
		std::list<EntityShape>::iterator ES;
		for(ES=Ent->mEntShapes.begin(); ES!=Ent->mEntShapes.end(); ES++)
		{
			if(Ent->miDmgTime==0)
			glColor3fv((*ES).Color.v);
			RenderCube((*ES).Pos,(*ES).Scale);
		}
		glEnd();
		if(Ent->mType == ENT_LASER)
		{
			UTIL_GL::SetBlend(GL_ONE,GL_ONE);
			glColor3f(0.5f,0,0);
			//glColor3fv((Ent->mEntShapes.front().Color*0.5f).v);
			glDisable(GL_LIGHTING);
			glBegin(GL_QUADS);
			RenderQuad(Vec3(0,0,0),4,mSpr.GetTileUV(24),mSpr.mUVsize,0);
			glEnd();
			glDisable(GL_BLEND);
			glEnable(GL_LIGHTING);
		}

		glPopMatrix();
	}


	glDisable(GL_LIGHTING);
	glDisable(GL_COLOR_MATERIAL);

	glDepthMask(GL_FALSE);
	if(!mGameFX.empty())
	{
		UTIL_GL::SetBlend(GL_ONE,GL_ONE);
		UTIL_GL::BindTexture(GL_TEXTURE_2D, mSprImg);
		glBegin(GL_QUADS);
		std::list<Effect*>::iterator FX;
		for(FX=mGameFX.begin(); FX!=mGameFX.end(); FX++)
		{
			(*FX)->Draw();
		}
		glEnd();
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_BLEND);
	}//*/
	glDepthMask(GL_TRUE);
	glDisable(GL_FOG);

	glDisable(GL_DEPTH_TEST);
	if(miPlayerDeathTime<=0)
	{
	UTIL_GL::BindTexture(GL_TEXTURE_2D, mSprImg);
	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glPushMatrix();
	glTranslatef(MouseX-320-mScroll.x,360-MouseY-mScroll.y,-70);
	glBegin(GL_QUADS);
	if(((miInternalClock/10)%2)==0)
		glColor3f(1,1,0);
	else
		glColor3f(1,0,0);
	RenderQuad(Vec3(0,0,0),16,mSpr.GetTileUV(56)+Vec2(0.005,0.005),mSpr.mUVsize-Vec2(0.01,0.01),0);
	glEnd();
	glPopMatrix();
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	}


	UTIL_GL::GL2D::SetOrtho(320,180);
	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(0,0,0,0.75f);
	glBegin(GL_QUADS);
	glVertex2f(0,0); glVertex2f(320,0);
	glVertex2f(320,15); glVertex2f(0,15);

	glVertex2f(0,165); glVertex2f(320,165);
	glVertex2f(320,180); glVertex2f(0,180);
	glEnd();
	mFont1.SetAlignment(ALIGN_LEFT);
	mFont1.SetColor(1,1,0.7f,1);
	char sz[64];
	//sprintf(sz,"FPS: %i", mTimer.GetFPS());
	//sprintf(sz,"TICKS: %i", miHomeRun);
	//mFont1.Print(sz,200,5);

	mFont1.Print("1UP", 10,5);
	sprintf(sz,"%.6i", miScore);
	mFont1.EnableFixedWidth();
	mFont1.SetFixedWidthSpacing(0.5f);
	mFont1.Print(sz, 40,5);
	mFont1.DisableFixedWidth();

	if(miScreenFlashTime>0)
	{
		float A = UTIL_Misc::TimeRamp(1,0,0,75,75-miScreenFlashTime);
		glColor4f(1,1,1,A);
		UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glBegin(GL_QUADS);
		glVertex2f(0,0); glVertex2f(320,0);
		glVertex2f(320,180); glVertex2f(0,180);
		glEnd();
		glDisable(GL_BLEND);
		glColor4f(1,1,1,1);
	}
	if(miPlayerDeathTime>0)
	{
		float A = UTIL_Misc::TimeRamp(0,1,0,200,200-miPlayerDeathTime);
		glColor4f(0,0,0,A);
		UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glBegin(GL_QUADS);
		glVertex2f(0,0); glVertex2f(320,0);
		glVertex2f(320,180); glVertex2f(0,180);
		glEnd();
		glDisable(GL_BLEND);
		glColor4f(1,1,1,1);
	}
	if(miInternalClock<0)
	{
		float Y = miInternalClock+200;
		glColor4f(0,0,0,1);
		glBegin(GL_QUADS);
		glVertex2f(0,0); glVertex2f(320,0);
		glVertex2f(320,90-Y); glVertex2f(0,90-Y);
		glVertex2f(0,90+Y); glVertex2f(320,90+Y);
		glVertex2f(320,180); glVertex2f(0,180);
		glEnd();
		glColor4f(1,1,1,1);
	}

	std::list<FontWriter>::iterator FW;
	for(FW=mFontW.begin(); FW!=mFontW.end(); FW++)
		(*FW).Render(interp);

}
void Engine::GiveUpgrade()
{
	miSegments++;
	if(miSegments>4)
	{
		miSegments = 4;
		return;
	}

	btTransform trans;
	mPlrBody->getMotionState()->getWorldTransform(trans);
	btVector3 btP = trans.getOrigin();

	btVector3 boxinertia;
	mBoxShape->calculateLocalInertia(5.5f,boxinertia);
	Vec3 boxinert(boxinertia.x(),boxinertia.y(),boxinertia.z());

	Entity* StatEnt = new Entity(EntityShape(Vec3(3.5f,3.5f,3.5f), Vec3(1,1,0)), ENT_PLAYER);
	btRigidBody* RB = InsertNewRigidBody(mBoxShape,Vec3(0,0,0),
	Vec3(btP.x(),btP.y()+(miSegments)*15,-70),
	1, boxinert,StatEnt, COL_PLR, COL_ENTITY | COL_STATIC);
	mGameEnts.push_back(StatEnt);

	mPlrBodySegs[miSegments-1] = RB;
	mPlrBodySegs[miSegments-1]->setGravity(btVector3(0,0,0));
	mPlrBodySegs[miSegments-1]->setAngularFactor(btVector3(0,0,0));

}
/**************************************************************************/
/* initialize extensions												  */
/**************************************************************************/
bool Engine::InitExtensions()
{
	gLog.OutPut("\n[Initializing Extensions]\n");
	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		gLog.OutPut("Failed To Init GLEW!\n");
		return false;
	}
	//fragment programs
	gLog.OutPut("Testing GL_ARB_depth_texture support.\n");
	if (!glewIsSupported("GL_ARB_depth_texture"))
	{
		gLog.OutPut("GL_ARB_depth_texture not supported!\n");
		return false;
	}
	gLog.OutPut("GL_ARB_depth_texture supported!\n");
	//fbo's
	gLog.OutPut("Testing GL_EXT_framebuffer_object support.\n");
	if (!glewIsSupported("GL_EXT_framebuffer_object"))
	{
		gLog.OutPut("GL_EXT_framebuffer_object not supported!\n");
		return false;
	}
	gLog.OutPut("GL_EXT_framebuffer_object supported!\n");


	if(gVsync)
	{
		wglSwapIntervalEXT(1);
		gLog.OutPut("Vsync enabled.\n");
	}
	else
	{
		wglSwapIntervalEXT(0);
		gLog.OutPut("Vsync disabled.\n");
	}
	return true;
}
/**************************************************************************/
/* Initialize shaders													  */
/**************************************************************************/
#include "Shaders.h"
bool Engine::InitShaders()
{
	gLog.OutPut("\n[Compilling Shaders]\n");
	std::stringstream shdr;
	std::string str;

	//***PASS***
	if(!LoadShader("pass",vp_pass, fp_pass, shdrPass))
		return false;
	if(!LoadShader("pass tex",vp_passTex, fp_passTex, shdrPassTex))
		return false;
	if(!LoadShader("pass rect",vp_passRect, fp_passRect, shdrPassRect))
		return false;
	if(!LoadShader("pass shadow",vp_passShadowMap, fp_passShadowMap, shdrShadowMap))
		return false;
	if(!LoadShader("pass geopoint",vp_passGeo, gp_passPoint, fp_passGeo, shdrPassGeoPoint, GL_POINTS, GL_TRIANGLE_STRIP, 12))
		return false;
	if(!LoadShader("pass geopointModel",vp_passGeo, gp_passPointModel, fp_passGeo, shdrPassGeoPointModel, GL_POINTS, GL_TRIANGLE_STRIP, 12))
		return false;
	if(!LoadShader("pass geopointShadow",vp_passGeoShadow, gp_passPointShadow, fp_passGeoShadow, shdrPassGeoPointShadow, GL_POINTS, GL_TRIANGLE_STRIP, 12))
		return false;


	gLog.OutPut("Complete...\n\n");
	return true;
}
/**************************************************************************/
/* load a shader														  */
/**************************************************************************/
bool Engine::LoadShader(char* name, char* szVp, char* szFp, int& outShdr)
{
	gLog.OutPut("Compilling Shader: ");
	gLog.OutPut(name);
	gLog.OutPut("\n");
	unsigned int vp,fp;
	if(!(fp = mShaderMgr.CreateShader(GL_FRAGMENT_SHADER_ARB,szFp)))
		return false;
	if(!(vp = mShaderMgr.CreateShader(GL_VERTEX_SHADER_ARB,szVp)))
		return false;
	outShdr = mShaderMgr.CreateProgram();
	mShaderMgr.AttachShader(vp);
	mShaderMgr.AttachShader(fp);
	mShaderMgr.LinkProgram();
	mShaderMgr.BindProgram(0);
	gLog.OutPut("Complete.\n");
	return true;
}
bool Engine::LoadShader(char* name, char* szVp, char* szGp, char* szFp, int& outShdr, int intype, int outtype, int count)
{
	gLog.OutPut("Compilling Shader: ");
	gLog.OutPut(name);
	gLog.OutPut("\n");
	unsigned int vp,fp, gp;
	if(!(fp = mShaderMgr.CreateShader(GL_FRAGMENT_SHADER_ARB,szFp)))
		return false;
	if(!(gp = mShaderMgr.CreateShader(GL_GEOMETRY_SHADER_ARB,szGp)))
		return false;
	if(!(vp = mShaderMgr.CreateShader(GL_VERTEX_SHADER_ARB,szVp)))
		return false;
	outShdr = mShaderMgr.CreateProgram();
	mShaderMgr.SetGeometryPrimitive(outShdr,intype,outtype,count);
	mShaderMgr.AttachShader(vp);
	mShaderMgr.AttachShader(gp);
	mShaderMgr.AttachShader(fp);
	mShaderMgr.LinkProgram();
	mShaderMgr.BindProgram(0);
	gLog.OutPut("Complete.\n");
	return true;
}
/**************************************************************************/
/* intialize physics													  */
/**************************************************************************/
void Engine::InitializePhysics()
{
	gLog.OutPut("Initializing Physics Engine.\n");
	mBulletBroadphase = new btDbvtBroadphase();
	mBulletCollisionConfiguration = new btDefaultCollisionConfiguration();
	mBulletDispatcher = new btCollisionDispatcher(mBulletCollisionConfiguration);
	mBulletSolver = new btSequentialImpulseConstraintSolver;
	mBulletDynamicsWorld = new btDiscreteDynamicsWorld(mBulletDispatcher,mBulletBroadphase,mBulletSolver,mBulletCollisionConfiguration);
	mBulletDynamicsWorld->setGravity(btVector3(0,-5,0));
	InitScene();
}
void Engine::InitScene()
{
/*	btCollisionShape* groundShape = new btStaticPlaneShape(btVector3(0,1,0),1);
	Entity* StatEnt = new Entity(EntityShape(Vec3(1,1,5), Vec3(1,1,1)), ENT_STATIC);
	btRigidBody* RB = InsertNewRigidBody(groundShape,Vec3(0,0,0), Vec3(0,-150,0),0, Vec3(0,0,0), StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR );
	mGameEnts.push_back(StatEnt);
/*	btCollisionShape* groundShape2 = new btStaticPlaneShape(btVector3(1,0,0),1);
	StatEnt = new Entity(EntityShape(Vec3(1,1,5), Vec3(1,1,1)), ENT_STATIC);
	RB = InsertNewRigidBody(groundShape2,Vec3(0,0,0), Vec3(-200,0,0),0, Vec3(0,0,0), StatEnt);
	mGameEnts.push_back(StatEnt);
	StatEnt = new Entity(EntityShape(Vec3(1,1,5), Vec3(1,1,1)), ENT_STATIC);
	RB = InsertNewRigidBody(groundShape2,Vec3(180,0,0), Vec3(200,0,0),0, Vec3(0,0,0), StatEnt);
	mGameEnts.push_back(StatEnt);*/

//	btCollisionShape* groundShape3 = new btStaticPlaneShape(btVector3(0,0,1),1);
//	InsertNewRigidBody(groundShape3,Vec3(0,0,0), Vec3(0,0,-80),0, Vec3(0,0,0),0);
//	InsertNewRigidBody(groundShape3,Vec3(0,180,0), Vec3(0,0,-60),0, Vec3(0,0,0),0);

	btRigidBody* RB = NULL;

	mBoxShape = new btBoxShape(btVector3(5,5,5));
	mBoxShapeMed = new btBoxShape(btVector3(10,10,10));
	mBoxShapeBig = new btBoxShape(btVector3(20,10,10));
	btVector3 boxinertia;
	mBoxShape->calculateLocalInertia(5.5f,boxinertia);
	Vec3 boxinert(boxinertia.x(),boxinertia.y(),boxinertia.z());


/*	for(int n=0; n<10; n++)
	{
	btCollisionShape* wallShape = new btBoxShape(btVector3(5,1000,15));
	StatEnt = new Entity(EntityShape(Vec3(5,1000,15), Vec3(1,0,1)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(-250,0 + n*1000,-70),
	0, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	StatEnt = new Entity(EntityShape(Vec3(5,1000,15), Vec3(1,0,1)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(250,0 + n*1000,-70),
	0, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	}*/


/*	for(int n=0; n<100; n++)
		for(int x=0; x<10;x++)
	{
		Entity* Ent = new Entity( EntityShape(Vec3(5,5,5), Vec3(0.6f,0.6f,0.6f)), ENT_BOX);
		RB = InsertNewRigidBody(boxShape,Vec3(UTIL_Misc::Rand()%360,UTIL_Misc::Rand()%360,UTIL_Misc::Rand()%360),
		Vec3(-190 + x*11,15 + n*50,-70),
		1.0f, boxinert, Ent, COL_ENTITY, COL_ENTITY | COL_LASER | COL_PLR | COL_STATIC);
		mGameEnts.push_back(Ent);
	}*/

	//player
	Entity* StatEnt = new Entity(EntityShape(Vec3(5,5,5), Vec3(1,1,0)), ENT_PLAYER);
	RB = InsertNewRigidBody(mBoxShape,Vec3(0,0,0),
	Vec3(-550,180,-70),
	1, boxinert,StatEnt, COL_PLR, COL_ENTITY | COL_STATIC);
	mGameEnts.push_back(StatEnt);

	mPlrBody = RB;
	mPlrBody->setGravity(btVector3(0,0,0));
	mPlrBody->setAngularFactor(btVector3(0,0,0));

	//map 
	float C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	btCollisionShape* wallShape = new btBoxShape(btVector3(300/2,40/2,15));
	wallShape->calculateLocalInertia(2000.5f,boxinertia);
	boxinert = Vec3(boxinertia.x(),boxinertia.y(),boxinertia.z());

	StatEnt = new Entity(EntityShape(Vec3(300/2,40/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(150,360-20,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	wallShape = new btBoxShape(btVector3(110/2,80/2,15));
	StatEnt = new Entity(EntityShape(Vec3(110/2,80/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(355,360-40,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(430/2,30/2,15));
	StatEnt = new Entity(EntityShape(Vec3(430/2,30/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(285,360-345,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(380/2,40/2,15));
	StatEnt = new Entity(EntityShape(Vec3(380/2,40/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(690,360-20,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(90/2,70/2,15));
	StatEnt = new Entity(EntityShape(Vec3(90/2,70/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(635,360-325,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	wallShape = new btBoxShape(btVector3(310/2,30/2,15));
	StatEnt = new Entity(EntityShape(Vec3(310/2,30/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(825,360-345,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(50/2,150/2,15));
	StatEnt = new Entity(EntityShape(Vec3(50/2,150/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(895,360-155,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	wallShape = new btBoxShape(btVector3(200/2,30/2,15));
	StatEnt = new Entity(EntityShape(Vec3(200/2,30/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1020,360-215,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(100/2,100/2,15));
	StatEnt = new Entity(EntityShape(Vec3(100/2,100/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1290,360-50,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(270/2,50/2,15));
	StatEnt = new Entity(EntityShape(Vec3(270/2,50/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1305,360-335,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	wallShape = new btBoxShape(btVector3(80/2,50/2,15));
	StatEnt = new Entity(EntityShape(Vec3(80/2,50/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1370,360-285,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(100/2,80/2,15));
	StatEnt = new Entity(EntityShape(Vec3(100/2,80/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1540,360-170,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(90/2,50/2,15));
	StatEnt = new Entity(EntityShape(Vec3(90/2,50/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1635,360-25,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(50/2,90/2,15));
	StatEnt = new Entity(EntityShape(Vec3(50/2,90/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1685,360-315,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(100/2,80/2,15));
	StatEnt = new Entity(EntityShape(Vec3(100/2,80/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1850,360-120,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

//	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(40/2,40/2,15));
	StatEnt = new Entity(EntityShape(Vec3(40/2,40/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1850,360-170,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(190/2,40/2,15));
	StatEnt = new Entity(EntityShape(Vec3(190/2,40/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(1935,360-300,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

//	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(40/2,130/2,15));
	StatEnt = new Entity(EntityShape(Vec3(40/2,130/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2010,360-215,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(190/2,50/2,15));
	StatEnt = new Entity(EntityShape(Vec3(190/2,50/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2195,360-25,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

//	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(90/2,80/2,15));
	StatEnt = new Entity(EntityShape(Vec3(90/2,80/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2245,360-90,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(190/2,50/2,15));
	StatEnt = new Entity(EntityShape(Vec3(190/2,50/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2195,360-335,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

//	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(90/2,70/2,15));
	StatEnt = new Entity(EntityShape(Vec3(90/2,70/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2245,360-275,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(90/2,40/2,15));
	StatEnt = new Entity(EntityShape(Vec3(90/2,40/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2455,360-20,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

//	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(90/2,260/2,15));
	StatEnt = new Entity(EntityShape(Vec3(90/2,260/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2455,360-230,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(90/2,210/2,15));
	StatEnt = new Entity(EntityShape(Vec3(90/2,210/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2675,360-105,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

//	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(300/2,40/2,15));
	StatEnt = new Entity(EntityShape(Vec3(300/2,40/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2870,360-110,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(580/2,30/2,15));
	StatEnt = new Entity(EntityShape(Vec3(580/2,30/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(2930,360-345,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(70/2,240/2,15));
	StatEnt = new Entity(EntityShape(Vec3(70/2,240/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(3185,360-210,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));


	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(580/2,30/2,15));
	StatEnt = new Entity(EntityShape(Vec3(580/2,30/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(5030,360-345,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(580/2,30/2,15));
	StatEnt = new Entity(EntityShape(Vec3(580/2,30/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(5030,360-15,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));


	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(80/2,80/2,15));
	StatEnt = new Entity(EntityShape(Vec3(80/2,80/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(5030,360-180,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(80/2,80/2,15));
	StatEnt = new Entity(EntityShape(Vec3(80/2,80/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(5230,360-240,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(80/2,80/2,15));
	StatEnt = new Entity(EntityShape(Vec3(80/2,80/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(5430,360-100,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(80/2,120/2,15));
	StatEnt = new Entity(EntityShape(Vec3(80/2,120/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(6030,360-60,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(80/2,120/2,15));
	StatEnt = new Entity(EntityShape(Vec3(80/2,120/2,15), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(6030,360-300,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(40/2,40/2,10));
	StatEnt = new Entity(EntityShape(Vec3(40/2,40/2,10), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(6030,360-240,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(40/2,40/2,10));
	StatEnt = new Entity(EntityShape(Vec3(40/2,40/2,10), Vec3(C,C,C)), ENT_STATIC);
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(6030,360-120,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));

	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	wallShape = new btBoxShape(btVector3(10,10,10));
	StatEnt = new Entity(EntityShape(Vec3(10,10,10), Vec3(1,0,0)), ENT_CORE);
	StatEnt->miLife = 2500;
	RB = InsertNewRigidBody(wallShape,Vec3(0,0,0),
	Vec3(6030,360-180,-70),
	2000, boxinert,StatEnt, COL_STATIC, COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,0));
	
	RB->setGravity(btVector3(0,0,0));
	/**************************************************************************/
	//specials
	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	btCompoundShape* compound = new btCompoundShape(true);
	wallShape = new btBoxShape(btVector3(250/2,20/2,15));
	btTransform trans;
	trans.setIdentity();
	trans.setOrigin(btVector3(0,0,0));
	compound->addChildShape(trans, wallShape);
	wallShape = new btBoxShape(btVector3(20/2,250/2,15));
	compound->addChildShape(trans, wallShape);

	btVector3 wallinertia;
	mBoxShape->calculateLocalInertia(1000.0f,wallinertia);
	Vec3 wallinert(wallinertia.x(),wallinertia.y(),wallinertia.z());

	StatEnt = new Entity(EntityShape(Vec3(250/2,20/2,15), Vec3(C,C,C)), ENT_STATIC);
	StatEnt->mEntShapes.push_back(EntityShape(Vec3(20/2,250/2,15), Vec3(C,C,C)));
	RB = InsertNewRigidBody(compound,Vec3(0,0,0),
	Vec3(3500,360-180,-70),
	5000, wallinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,1));
	RB->setAngularVelocity(btVector3(0,0,5));
	RB->setGravity(btVector3(0,0,0));
	//RB->applyCentralImpulse(btVector3(0,-100,0));

	StatEnt = new Entity(EntityShape(Vec3(250/2,20/2,15), Vec3(C,C,C)), ENT_STATIC);
	StatEnt->mEntShapes.push_back(EntityShape(Vec3(20/2,250/2,15), Vec3(C,C,C)));
	RB = InsertNewRigidBody(compound,Vec3(0,0,0),
	Vec3(4000,360-80,-70),
	5000, wallinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,1));
	RB->setAngularVelocity(btVector3(0,0,5));
	RB->setGravity(btVector3(0,0,0));

	StatEnt = new Entity(EntityShape(Vec3(250/2,20/2,15), Vec3(C,C,C)), ENT_STATIC);
	StatEnt->mEntShapes.push_back(EntityShape(Vec3(20/2,250/2,15), Vec3(C,C,C)));
	RB = InsertNewRigidBody(compound,Vec3(0,0,0),
	Vec3(4300,360-280,-70),
	5000, wallinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,1));
	RB->setAngularVelocity(btVector3(0,0,5));
	RB->setGravity(btVector3(0,0,0));


	compound = new btCompoundShape(true);
	wallShape = new btBoxShape(btVector3(130/2,20/2,15));
	trans.setIdentity();
	trans.setOrigin(btVector3(0,0,0));
	compound->addChildShape(trans, wallShape);
	wallShape = new btBoxShape(btVector3(20/2,130/2,15));
	compound->addChildShape(trans, wallShape);

	StatEnt = new Entity(EntityShape(Vec3(130/2,20/2,15), Vec3(C,C,C)), ENT_STATIC);
	StatEnt->mEntShapes.push_back(EntityShape(Vec3(20/2,130/2,15), Vec3(C,C,C)));
	RB = InsertNewRigidBody(compound,Vec3(0,0,0),
	Vec3(3750,360-80,-70),
	5000, wallinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,1));
	RB->setAngularVelocity(btVector3(0,0,5));
	RB->setGravity(btVector3(0,0,0));

	StatEnt = new Entity(EntityShape(Vec3(130/2,20/2,15), Vec3(C,C,C)), ENT_STATIC);
	StatEnt->mEntShapes.push_back(EntityShape(Vec3(20/2,130/2,15), Vec3(C,C,C)));
	RB = InsertNewRigidBody(compound,Vec3(0,0,0),
	Vec3(3950,360-280,-70),
	5000, wallinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,1));
	RB->setAngularVelocity(btVector3(0,0,5));
	RB->setGravity(btVector3(0,0,0));

	StatEnt = new Entity(EntityShape(Vec3(130/2,20/2,15), Vec3(C,C,C)), ENT_STATIC);
	StatEnt->mEntShapes.push_back(EntityShape(Vec3(20/2,130/2,15), Vec3(C,C,C)));
	RB = InsertNewRigidBody(compound,Vec3(0,0,0),
	Vec3(4250,360-80,-70),
	5000, wallinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,0,0));
	RB->setAngularFactor(btVector3(0,0,1));
	RB->setAngularVelocity(btVector3(0,0,5));
	RB->setGravity(btVector3(0,0,0));


	mTriggerShape1 = new btBoxShape(btVector3(15/2,50/2,15/2));
	C = 0.5f+UTIL_Misc::RandFloat(0.5f);
	compound = new btCompoundShape(true);
	trans.setIdentity();
	wallShape = new btBoxShape(btVector3(10/2,50/2,10/2));
	trans.setOrigin(btVector3(0,0,0));
	compound->addChildShape(trans, wallShape);
	wallShape = new btBoxShape(btVector3(110/2,30/2,15));
	trans.setOrigin(btVector3(0,0,0));
	compound->addChildShape(trans, wallShape);

	mBoxShape->calculateLocalInertia(5000.0f,wallinertia);
	wallinert = Vec3(wallinertia.x(),wallinertia.y(),wallinertia.z());

/*	StatEnt = new Entity(EntityShape(Vec3(110/2,30/2,15), Vec3(C,C,C)), ENT_STATIC);
	StatEnt->mEntShapes.push_back(EntityShape(Vec3(0,-20,0),Vec3(10/2,10/2,10/2), Vec3(1,0,0)));
	StatEnt->mbWarp = true;
	RB = InsertNewRigidBody(compound,Vec3(0,0,0),
	Vec3(2565,360-360,-70),
	1000, wallinert,StatEnt, COL_STATIC, COL_ENTITY | COL_LASER | COL_PLR);
	mGameEnts.push_back(StatEnt);
	RB->setLinearFactor(btVector3(0,1,0));
	RB->setAngularFactor(btVector3(0,0,0));
	RB->setGravity(btVector3(0,0,0));
	RB->setLinearVelocity(btVector3(0,50,0));
	//RB->applyCentralImpulse(btVector3(0,-100,0));*/

	//entities
	BoxGenerator* mGen1 = new BoxGenerator(EntityShape(Vec3(1,1,1),Vec3(1,1,1)),ENT_BOXGEN);
	mGen1->mForce = Vec3(-120,0,0);
	mGen1->mPos = Vec3(600-500,180,-70);
	mGen1->miRadius = 100;
	mGen1->miBoxFrequency = 3;
	mGen1->mbInfiniteBoxes = true;
	mGameEnts.push_back(mGen1);

	BoxGenerator* mGen2 = new BoxGenerator(EntityShape(Vec3(1,1,1),Vec3(1,1,1)),ENT_BOXGEN);
	mGen2->mForce = Vec3(0,-50,0);
	mGen2->mPos = Vec3(100-500,400,-70);
	mGen2->miRadius = 100;
	mGen2->miBoxFrequency = 3;
	mGen2->mbInfiniteBoxes = true;
	mGameEnts.push_back(mGen2);


	BoxGenerator* mGen3 = new BoxGenerator(EntityShape(Vec3(1,1,1),Vec3(1,1,1)),ENT_BOXGEN);
	mGen3->mForce = Vec3(60,30,0);
	mGen3->mPos = Vec3(-350-500,180,-70);
	mGen3->miRadius = 100;
	mGen3->miBoxFrequency = 5;
	mGen3->mbInfiniteBoxes = true;
	mGameEnts.push_back(mGen3);


	CompoundGenerator* mGen4 = new CompoundGenerator(EntityShape(Vec3(1,1,1),Vec3(1,1,1)),ENT_BOXGEN,compound,wallinert,5000);
	mGen4->mComShapes.push_back(EntityShape(Vec3(0,0,0),Vec3(10/2,50/2,10/2), Vec3(1,0,0)));
	mGen4->mComShapes.push_back(EntityShape(Vec3(110/2,30/2,15), Vec3(C,C,C)));
	mGen4->mForce = Vec3(0,50,0);
	mGen4->mPos = Vec3(2350,360-400,-70);
	mGen4->miRadius = 1;
	mGen4->miBoxFrequency = 300;
	mGen4->mbInfiniteBoxes = true;
	mGameEnts.push_back(mGen4);

	CompoundGenerator* mGen5 = new CompoundGenerator(EntityShape(Vec3(1,1,1),Vec3(1,1,1)),ENT_BOXGEN,compound,wallinert,5000);
	mGen5->mComShapes.push_back(EntityShape(Vec3(0,0,0),Vec3(10/2,50/2,10/2), Vec3(1,0,0)));
	mGen5->mComShapes.push_back(EntityShape(Vec3(110/2,30/2,15), Vec3(C,C,C)));
	mGen5->mForce = Vec3(0,-20,0);
	mGen5->mPos = Vec3(2565,400,-70);
	mGen5->miRadius = 1;
	mGen5->miBoxFrequency = 600;
	mGen5->mbInfiniteBoxes = true;
	mGameEnts.push_back(mGen5);

}
/**************************************************************************/
/* shutdown physics														  */
/**************************************************************************/
void Engine::ShutDownPhysics()
{
	gLog.OutPut("Shutdown Physics Engine.\n");
	if(!mBulletRigidBodies.empty())
	{
		std::list<btRigidBody*>::iterator IT;
		for(IT=mBulletRigidBodies.begin(); IT!=mBulletRigidBodies.end(); IT++)
		{
			mBulletDynamicsWorld->removeRigidBody((*IT));
			delete (*IT)->getMotionState();
			delete (*IT);
		}
	}
	if(!mBulletShapes.empty())
	{
		std::list<btCollisionShape*>::iterator IT;
		for(IT=mBulletShapes.begin(); IT!=mBulletShapes.end(); IT++)
		{
			delete (*IT);
		}
	}
	delete mBulletDynamicsWorld;
	delete mBulletSolver;
	delete mBulletCollisionConfiguration;
	delete mBulletDispatcher;
	delete mBulletBroadphase;
}
/**************************************************************************/
/* update physics														  */
/**************************************************************************/
void Engine::UpdatePhysics()
{
	mBulletDynamicsWorld->stepSimulation(1,1);
}
/**************************************************************************/
/* insert rigid body													  */
/**************************************************************************/
btRigidBody* Engine::InsertNewRigidBody(btCollisionShape* Shape, Vec3 Angles, Vec3 Pos, float mass, Vec3 inertia, Entity* Ent, short int ColMask, short int ColWith)
{
	btDefaultMotionState* MotionState = new btDefaultMotionState(btTransform(btQuaternion(DEG2RAD(Angles.x),DEG2RAD(Angles.y),DEG2RAD(Angles.z)),btVector3(Pos.x,Pos.y,Pos.z)));
	btRigidBody::btRigidBodyConstructionInfo RigidBodyCI(mass,MotionState,Shape,btVector3(inertia.x,inertia.y,inertia.z));
	btRigidBody* RigidBody = new btRigidBody(RigidBodyCI);
	RigidBody->setLinearFactor(btVector3(1,1,0));
	RigidBody->setUserPointer((void*)Ent);
	mBulletDynamicsWorld->addRigidBody(RigidBody,ColMask, ColWith);
	mBulletRigidBodies.push_back(RigidBody);
	return RigidBody;
}
void Engine::RenderCube(float scale)
{
/*	glMultiTexCoord2s(GL_TEXTURE1,220,0);
	glNormal3f(scale,scale,scale);
	glVertex3f(0,0,0);
	glMultiTexCoord2s(GL_TEXTURE1,220,0);
	glNormal3f(-scale,-scale,-scale);
	glVertex3f(0,0,0);
	return;*/
//	glColor3f(1,0,0);
	glNormal3f(0,-1,0);
	glVertex3f(-scale,-scale,-scale);
	glVertex3f(scale,-scale,-scale);
	glVertex3f(scale,-scale,scale);
	glVertex3f(-scale,-scale,scale);
	glNormal3f(0,1,0);
	glVertex3f(scale,scale,-scale);
	glVertex3f(-scale,scale,-scale);
	glVertex3f(-scale,scale,scale);
	glVertex3f(scale,scale,scale);
//	glColor3f(0,1,0);
	glNormal3f(-1,0,0);
	glVertex3f(-scale,-scale,-scale);
	glVertex3f(-scale,-scale,scale);
	glVertex3f(-scale,scale,scale);
	glVertex3f(-scale,scale,-scale);
	glNormal3f(1,0,0);
	glVertex3f(scale,-scale,scale);
	glVertex3f(scale,-scale,-scale);
	glVertex3f(scale,scale,-scale);
	glVertex3f(scale,scale,scale);
//	glColor3f(0,0,1);
	glNormal3f(0,0,-1);
	glVertex3f(-scale,-scale,-scale);
	glVertex3f(scale,-scale,-scale);
	glVertex3f(scale,scale,-scale);
	glVertex3f(-scale,scale,-scale);
	glNormal3f(0,0,1);
	glVertex3f(scale,-scale,scale);
	glVertex3f(-scale,-scale,scale);
	glVertex3f(-scale,scale,scale);
	glVertex3f(scale,scale,scale);
}
void Engine::RenderCube(Vec3 scale)
{
/*	glMultiTexCoord2s(GL_TEXTURE1,220,0);
	glNormal3f(scale,scale,scale);
	glVertex3f(0,0,0);
	glMultiTexCoord2s(GL_TEXTURE1,220,0);
	glNormal3f(-scale,-scale,-scale);
	glVertex3f(0,0,0);
	return;*/
//	glColor3f(1,0,0);
	glNormal3f(0,-1,0);
	glVertex3f(-scale.x,-scale.y,-scale.z);
	glVertex3f(scale.x,-scale.y,-scale.z);
	glVertex3f(scale.x,-scale.y,scale.z);
	glVertex3f(-scale.x,-scale.y,scale.z);
	glNormal3f(0,1,0);
	glVertex3f(scale.x,scale.y,-scale.z);
	glVertex3f(-scale.x,scale.y,-scale.z);
	glVertex3f(-scale.x,scale.y,scale.z);
	glVertex3f(scale.x,scale.y,scale.z);
//	glColor3f(0,1,0);
	glNormal3f(-1,0,0);
	glVertex3f(-scale.x,-scale.y,-scale.z);
	glVertex3f(-scale.x,-scale.y,scale.z);
	glVertex3f(-scale.x,scale.y,scale.z);
	glVertex3f(-scale.x,scale.y,-scale.z);
	glNormal3f(1,0,0);
	glVertex3f(scale.x,-scale.y,scale.z);
	glVertex3f(scale.x,-scale.y,-scale.z);
	glVertex3f(scale.x,scale.y,-scale.z);
	glVertex3f(scale.x,scale.y,scale.z);
//	glColor3f(0,0,1);
	glNormal3f(0,0,-1);
	glVertex3f(-scale.x,-scale.y,-scale.z);
	glVertex3f(scale.x,-scale.y,-scale.z);
	glVertex3f(scale.x,scale.y,-scale.z);
	glVertex3f(-scale.x,scale.y,-scale.z);
	glNormal3f(0,0,1);
	glVertex3f(scale.x,-scale.y,scale.z);
	glVertex3f(-scale.x,-scale.y,scale.z);
	glVertex3f(-scale.x,scale.y,scale.z);
	glVertex3f(scale.x,scale.y,scale.z);
}
void Engine::RenderCube(Vec3 P,float scale)
{
/*	glMultiTexCoord2s(GL_TEXTURE1,220,0);
	glNormal3f(scale,scale,scale);
	glVertex3f(0,0,0);
	glMultiTexCoord2s(GL_TEXTURE1,220,0);
	glNormal3f(-scale,-scale,-scale);
	glVertex3f(0,0,0);
	return;*/
//	glColor3f(1,0,0);
	glNormal3f(0,-1,0);
	glVertex3f(P.x-scale,P.y-scale,P.z-scale);
	glVertex3f(P.x+scale,P.y-scale,P.z-scale);
	glVertex3f(P.x+scale,P.y-scale,P.z+scale);
	glVertex3f(P.x-scale,P.y-scale,P.z+scale);
	glNormal3f(0,1,0);
	glVertex3f(P.x+scale,P.y+scale,P.z-scale);
	glVertex3f(P.x-scale,P.y+scale,P.z-scale);
	glVertex3f(P.x-scale,P.y+scale,P.z+scale);
	glVertex3f(P.x+scale,P.y+scale,P.z+scale);
//	glColor3f(0,1,0);
	glNormal3f(-1,0,0);
	glVertex3f(P.x-scale,P.y-scale,P.z-scale);
	glVertex3f(P.x-scale,P.y-scale,P.z+scale);
	glVertex3f(P.x-scale,P.y+scale,P.z+scale);
	glVertex3f(P.x-scale,P.y+scale,P.z-scale);
	glNormal3f(1,0,0);
	glVertex3f(P.x+scale,P.y-scale,P.z+scale);
	glVertex3f(P.x+scale,P.y-scale,P.z-scale);
	glVertex3f(P.x+scale,P.y+scale,P.z-scale);
	glVertex3f(P.x+scale,P.y+scale,P.z+scale);
//	glColor3f(0,0,1);
	glNormal3f(0,0,-1);
	glVertex3f(P.x-scale,P.y-scale,P.z-scale);
	glVertex3f(P.x+scale,P.y-scale,P.z-scale);
	glVertex3f(P.x+scale,P.y+scale,P.z-scale);
	glVertex3f(P.x-scale,P.y+scale,P.z-scale);
	glNormal3f(0,0,1);
	glVertex3f(P.x+scale,P.y-scale,P.z+scale);
	glVertex3f(P.x-scale,P.y-scale,P.z+scale);
	glVertex3f(P.x-scale,P.y+scale,P.z+scale);
	glVertex3f(P.x+scale,P.y+scale,P.z+scale);
}
void Engine::RenderCube(Vec3 P,Vec3 scale)
{
	glNormal3f(0,-1,0);
	glVertex3f(P.x-scale.x,P.y-scale.y,P.z-scale.z);
	glVertex3f(P.x+scale.x,P.y-scale.y,P.z-scale.z);
	glVertex3f(P.x+scale.x,P.y-scale.y,P.z+scale.z);
	glVertex3f(P.x-scale.x,P.y-scale.y,P.z+scale.z);
	glNormal3f(0,1,0);
	glVertex3f(P.x+scale.x,P.y+scale.y,P.z-scale.z);
	glVertex3f(P.x-scale.x,P.y+scale.y,P.z-scale.z);
	glVertex3f(P.x-scale.x,P.y+scale.y,P.z+scale.z);
	glVertex3f(P.x+scale.x,P.y+scale.y,P.z+scale.z);
//	glColor3f(0,1,0);
	glNormal3f(-1,0,0);
	glVertex3f(P.x-scale.x,P.y-scale.y,P.z-scale.z);
	glVertex3f(P.x-scale.x,P.y-scale.y,P.z+scale.z);
	glVertex3f(P.x-scale.x,P.y+scale.y,P.z+scale.z);
	glVertex3f(P.x-scale.x,P.y+scale.y,P.z-scale.z);
	glNormal3f(1,0,0);
	glVertex3f(P.x+scale.x,P.y-scale.y,P.z+scale.z);
	glVertex3f(P.x+scale.x,P.y-scale.y,P.z-scale.z);
	glVertex3f(P.x+scale.x,P.y+scale.y,P.z-scale.z);
	glVertex3f(P.x+scale.x,P.y+scale.y,P.z+scale.z);
//	glColor3f(0,0,1);
	glNormal3f(0,0,-1);
	glVertex3f(P.x-scale.x,P.y-scale.y,P.z-scale.z);
	glVertex3f(P.x+scale.x,P.y-scale.y,P.z-scale.z);
	glVertex3f(P.x+scale.x,P.y+scale.y,P.z-scale.z);
	glVertex3f(P.x-scale.x,P.y+scale.y,P.z-scale.z);
	glNormal3f(0,0,1);
	glVertex3f(P.x+scale.x,P.y-scale.y,P.z+scale.z);
	glVertex3f(P.x-scale.x,P.y-scale.y,P.z+scale.z);
	glVertex3f(P.x-scale.x,P.y+scale.y,P.z+scale.z);
	glVertex3f(P.x+scale.x,P.y+scale.y,P.z+scale.z);
}
void Engine::RenderQuad(Vec3 P, float scale, Vec2 uv1, Vec2 uv2, float Angle)
{
	Vec2 Up, Rgt;
	UTIL_Misc::MakeVectors(Angle,Up,Rgt);
	Vec3 U(Up.x,Up.y,0);
	Vec3 R(Rgt.x,Rgt.y,0);
	glTexCoord2f(uv1.x, uv1.y+uv2.y);		glVertex3fv((P-U*scale-R*scale).v);
	glTexCoord2f(uv1.x+uv2.x, uv1.y+uv2.y);	glVertex3fv((P+U*scale-R*scale).v);
	glTexCoord2f(uv1.x+uv2.x, uv1.y);		glVertex3fv((P+U*scale+R*scale).v);
	glTexCoord2f(uv1.x, uv1.y);				glVertex3fv((P-U*scale+R*scale).v);
}