#pragma once
#include "UTIL_SDL.h"
#include "UTIL_OpenGL.h"
#include "UTIL_Misc.h"
#include "Timer.h"
#include "FileIO.h"
#include "Button.h"
#include "Log.h"
#include "Vector.h"
#include "Font.h"
#include <vector>
#include <list>
#include <string>
#include <algorithm>
#include "FrameBufferObject.h"
#include "SpriteSheet.h"
#include "ShaderMgr.h"
#include "Spline.h"
#include "Sound.h"
#include "FontWriter.h"
#include "Frustum.h"
#include "Entity.h"
#include "Effects.h"


const float TIME_STEP = 0.02f;
#define VOX_W 320
#define VOX_H 1008
#define VOX_D 128

enum ASPECT_RATIO
{
	RATIO_16_9 = 0,
	RATIO_4_3,
	RATIO_16_10,
	RATIO_5_4
};

enum SPLINE_NAMES
{
	FIRST_SPLINE = 0,

	MAX_SPLINES = 32
};


class Engine
{
public:
	double accumulator;

	Engine(int w, int h, bool fscreen);
	~Engine();
	int winWidth, winHeight, AspectRatio;
	int hudWidth, hudHeight;
	void InitDevil();
	void Pump();

	void Update();
	void Render(const float interp);

	void KeyStateAlltered(int key, bool state);
	void TextInput(SDL_keysym key);
	void KeysPostUpdate();
	void CheckJoystickMotion(SDL_Event event);
	void SaveInput();
	void LoadInput();
	Button KEY_MOUSE1, KEY_MOUSE2, KEY_MOUSE3, KEY_QUIT, KEY_PAUSE;
	Button KEY_LEFT, KEY_RIGHT, KEY_UP, KEY_DOWN, KEY_ATTACK1, KEY_ATTACK2;

	Button* pCfgButton;
	int iControlMethod;
	int MouseX, MouseY;
	bool bTextInput;
	char szInputText[6];
	int iTextIdx, iTextMaxLength;
	Vec2 mMouseMotion;
	float mfMouseSensi;
	Vec2 mGamepadMotion;
	int iOption;
	float fJoystickSensi;
	bool bCfgJoystick;
	int oldPadAxisH, oldPadAxisV;

	bool mbQuit,mbPaused;
	Timer mTimer;
	float mInterp;

	//font related
	Font mFont1;
	Font mFont2;
	Font mFont3;
	std::list<FontWriter> mFontW;

	int miInternalClock;
	//extension related
	bool InitExtensions();
	FBO mFBO;
	Vec2 MapWindowToInternal(int x, int y);


	//shader related
	ShaderMgr mShaderMgr;
	bool InitShaders();
	bool LoadShader(char* name, char* szVp, char* szFp, int& outShdr);
	bool LoadShader(char* name, char* szVp, char* szGp, char* szFp, int& outShdr, int intype, int outtype, int count);
	int shdrPass, shdrPassTex, shdrPassRect, shdrPassGeoPoint, shdrPassGeoPointModel, shdrPassGeoPointShadow, shdrShadowMap;

	//model & splines
	Spline mSplines[MAX_SPLINES];
	unsigned int mSprImg;
	SpriteSheet mSpr,mSpr2;

	//music related
	Music mMusic;

	//physics and collision related
	btBroadphaseInterface* mBulletBroadphase;
	btDefaultCollisionConfiguration* mBulletCollisionConfiguration;
	btCollisionDispatcher* mBulletDispatcher;
	btSequentialImpulseConstraintSolver* mBulletSolver;
	btDiscreteDynamicsWorld* mBulletDynamicsWorld;
	std::list<btCollisionShape*> mBulletShapes;
	std::list<btRigidBody*> mBulletRigidBodies;
	void InitializePhysics();
	void ShutDownPhysics();
	void UpdatePhysics();
	btRigidBody* InsertNewRigidBody(btCollisionShape* Shape, Vec3 Angles, Vec3 Pos, float mass, Vec3 inertia, Entity* Ent, short int ColMask, short int ColWith);
	void RenderCube(float scale);
	void RenderCube(Vec3 scale);
	void RenderCube(Vec3 P, float scale);
	void RenderCube(Vec3 P, Vec3 scale);
	void RenderQuad(Vec3 P, float scale, Vec2 uv1, Vec2 uv2, float Angle);


	//render related
	Frustum mFrustum;

	//camera related
	Vec3 mCamAngles;
	Vec3 mCameraPos, mCameraOPos;

	//Entities;
	std::list<Entity*> mGameEnts;
	std::list<Effect*> mGameFX;

	//player
	btRigidBody* mPlrBody;
	btRigidBody* mPlrBodySegs[8];
	int miSegments;
	int miTakeDamageTime;
	void GiveUpgrade();
	int miScore;
	bool mbCoreDestroyed;
	int miScreenFlashTime;
	int miPlayerDeathTime;
	int miHomeRun;
	void ResetGame();
	void InitScene();
	Vec2 mVibration;

	Vec3 mScroll, mScrollVel;
	//editor related
	bool mbUseEditor;
	Button KEY_EDIT_LEFT, KEY_EDIT_RIGHT, KEY_EDIT_UP, KEY_EDIT_DOWN,
		KEY_EDIT_ROTATE, KEY_EDIT_ZOOM_IN, KEY_EDIT_ZOOM_OUT, KEY_TOGGLE_EDITOR;

	//physics
	btCollisionShape* mBoxShape, *mBoxShapeMed, *mBoxShapeBig, *mTriggerShape1;
};
bool PruneRigidBody(btRigidBody* body);
bool PruneGameEntity(Entity* ent);
bool PruneFX(Effect* FX);

extern Engine* gpEng;