#pragma once
#include "Vector.h"
#include "UTIL_Misc.h"

#include <list>
#include <btBulletDynamicsCommon.h>
#include <btBulletCollisionCommon.h>
#define BIT(x) (1<<(x))
enum collisiontypes {
    COL_NOTHING = 0, 
    COL_PLR = BIT(0), 
    COL_LASER = BIT(1), 
    COL_ENTITY = BIT(2),
	COL_STATIC = BIT(3)
};

#define ENT_STATIC 0
#define ENT_PLAYER 1
#define ENT_LASER 2
#define ENT_BOX 10
#define ENT_MEDBOX 11
#define ENT_BOXGEN 20
#define ENT_COMGEN 21
#define ENT_TRIGGER 50
#define ENT_CORE 100

class EntityShape
{
public:
	Vec3 Scale;
	Vec3 Color;
	Vec3 Pos;
	EntityShape::EntityShape(Vec3 scale, Vec3 color)
	{
		Scale = scale;
		Color = color;
	}
	EntityShape::EntityShape(Vec3 P, Vec3 scale, Vec3 color)
	{
		Pos = P;
		Scale = scale;
		Color = color;
	}

};
class Entity
{
public:
	int miLife;
	int mType;
	bool mbJustSpawned;//stop boxes getting stuck inside other stuff
	std::list<EntityShape> mEntShapes;
	bool mRemove, mbTagged;
	int miDmgTime;
	btRigidBody* pBody;
	int miScoreValue;
	Entity* pOwner;
	Entity::Entity(EntityShape shape, int type)
	{
		mEntShapes.push_back(shape);
		mType = type;
		mRemove = false;
		miLife = 10;
		mbJustSpawned = true;
		miDmgTime = 0;
		miScoreValue = 25;
		pOwner = NULL;
		mbTagged = false;
	}
	virtual void Update();

	virtual void TakeDamage(int amount);

};
class Laser : public Entity
{
public:
	Laser::Laser(EntityShape shape, int type) : Entity(shape,type)
	{
		miLife = 100;
	}
	void Update()
	{
		miLife--;
		if(miLife<=0)
			mRemove = true;

		mEntShapes.front().Color = Vec3(0,UTIL_Misc::RandFloat(1),1);
	}
};

class Trigger : public Entity
{
public:
	Entity* pTarget;
	Trigger::Trigger(EntityShape shape, int type) : Entity(shape,type)
	{
		miLife = 150;
	}
};
class BoxGenerator : public Entity
{
public:
	bool mbInfiniteBoxes;
	int miBoxCount;
	int miMaxBoxes;
	int miBoxFrequency;
	int miFrequency;
	int miRadius;
	Vec3 mPos;
	Vec3 mForce;//boxes start with this force
	bool mbScroll;//move with scroll;
	bool bBigBoxes;

	BoxGenerator::BoxGenerator(EntityShape shape, int type) : Entity(shape, type)
	{
		miBoxCount = 0;
		miMaxBoxes = 500;
		miBoxFrequency = 5;
		miFrequency = 0;
		miRadius = 50;
		mbScroll = true;
		mbInfiniteBoxes = false;
		bBigBoxes = false;
	}
	void Update();
};

class CompoundGenerator : public Entity
{
public:
	bool mbInfiniteBoxes;
	int miBoxCount;
	int miMaxBoxes;
	int miBoxFrequency;
	int miFrequency;
	int miRadius;
	Vec3 mPos;
	Vec3 mForce;//boxes start with this force
	bool mbScroll;//move with scroll;
	btCompoundShape* pCompound;
	Vec3 inertia;
	float mMass;
	std::list<EntityShape> mComShapes;


	CompoundGenerator::CompoundGenerator(EntityShape shape, int type, btCompoundShape* pcom, Vec3 iner, float mass) : Entity(shape, type)
	{
		miBoxCount = 0;
		miMaxBoxes = 1;
		miBoxFrequency = 50;
		miFrequency = 0;
		miRadius = 50;
		mbScroll = false;
		mbInfiniteBoxes = false;
		pCompound = pcom;
		inertia = iner;
		mMass = mass;
	}
	void Update();
};