#include "FrameBufferObject.h"
#include "Engine.h"
#include "UTIL_openGL.h"

extern bool gAntiAliasing;
extern int gAASamples;

bool InitializeFBO(FBO& fbo, char* name, unsigned int w, unsigned int h, bool depth, bool stencil, bool color, bool depthTex, bool mipmaps, const unsigned int textype, int numCol)
{
	gLog.OutPut("Initializing FBO [");	gLog.OutPut(name);	gLog.OutPut("]\n");
	//initialize FBO
	strcpy(fbo.name, name);
	fbo.colorID[0] = 0;
	fbo.colorID[1] = 0;
	fbo.colortexID[0] = 0;
	fbo.colortexID[1] = 0;
	fbo.depthID = 0;
	fbo.depthtexID = 0;
	fbo.fboID = 0;
	fbo.stencilID = 0;
	fbo.w = w;
	fbo.h = h;
	fbo.numColors = numCol;
	
	//create fbo
	glGenFramebuffersEXT (1, &fbo.fboID);
	glBindFramebufferEXT (GL_FRAMEBUFFER_EXT, fbo.fboID);
	//build color buffer and texture
	if(color)
	{
		for(int n=0; n< numCol; n++)
		{
			gLog.OutPut("Generating color texture...");
			glGenTextures (1, &fbo.colortexID[n]);
			glBindTexture (textype, fbo.colortexID[n]);
			glTexImage2D (textype, 0, GL_RGBA8, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
			glTexParameteri(textype, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexParameteri(textype, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
			if(mipmaps)
			{
				glGenerateMipmapEXT(GL_TEXTURE_2D);
			}
			else
			{
				glTexParameteri(textype, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
				glTexParameteri(textype, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			}
			gLog.OutPut("Completed.\n");

			gLog.OutPut("Generating FBO color buffer...\n");
			glGenRenderbuffersEXT(1, &fbo.colorID[n]);
			glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, fbo.colorID[n]);
/*			if(gAntiAliasing)
			{
				glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, gAASamples, GL_RGBA8, w, h);
			}
			else*/
				glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_RGBA8, w, h);

			glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT+n, GL_RENDERBUFFER_EXT, fbo.colorID[n]);
			glBindFramebufferEXT (GL_FRAMEBUFFER_EXT, fbo.fboID);
			glFramebufferTexture2DEXT (GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT+n, textype, fbo.colortexID[n], 0);
			gLog.OutPut("Completed.\n");
		}
		if(!CheckFboStatus())
			return false;
	}

	//build depth and texture
	if(depth)
	{
		if(depthTex)
		{
			gLog.OutPut("Generating depth texture...");
			glGenTextures (1, &fbo.depthtexID);
			glBindTexture (textype, fbo.depthtexID);
//			if(stencil)
//				glTexImage2D (textype, 0, GL_DEPTH24_STENCIL8_EXT, w, h, 0, GL_DEPTH_STENCIL_EXT, GL_UNSIGNED_INT_24_8_EXT, NULL);
//			else
				glTexImage2D (textype, 0, GL_DEPTH_COMPONENT32, w, h, 0, GL_DEPTH_COMPONENT, GL_UNSIGNED_BYTE, NULL);
			UTIL_GL::TextureFilter(GL_NEAREST, GL_NEAREST);
			UTIL_GL::TextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);//*/
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_R_TO_TEXTURE);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, GL_LEQUAL);
			glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_INTENSITY);
			gLog.OutPut("Completed.\n");
		}

		gLog.OutPut("Generating FBO depth buffer...\n");
//		glGenRenderbuffersEXT(1, &fbo.depthID);
//		glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, fbo.depthID);
		if(stencil)
		{
/*			if(gAntiAliasing)
			{
				glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, gAASamples, GL_DEPTH24_STENCIL8_EXT, w, h);
			}
			else*/
//				glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH24_STENCIL8_EXT, w, h);
		}
//		glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, fbo.depthID);


		if(depthTex)
		{
//			if(stencil)
//				glFramebufferTexture2DEXT (GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT, textype, fbo.depthtexID, 0);
//			else
				glFramebufferTexture2DEXT (GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, textype, fbo.depthtexID, 0);
		}

		gLog.OutPut("Completed.\n");
		if(!CheckFboStatus())
			return false;
	}
	//build stencil buffer
	if(stencil)
	{
		gLog.OutPut("Generating FBO stencil buffer...\n");
		//glGenRenderbuffersEXT(1, &fbo.stencilID);
		//glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, fbo.stencilID);
		//glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_STENCIL_INDEX, w, h);
//		glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, fbo.depthID);
//		glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, fbo.depthID);
		if(depthTex)
			glFramebufferTexture2DEXT (GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT, textype, fbo.depthtexID, 0);
		gLog.OutPut("Completed.\n");
		if(!CheckFboStatus())
			return false;
	}

	gLog.OutPut("FBO initialized successfully.\n\n");
//	glEnable(GL_MULTISAMPLE_ARB);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);

	return true;
}

bool CheckFboStatus()
{
	gLog.OutPut("Checking FBO status...");
	GLenum status = glCheckFramebufferStatusEXT (GL_FRAMEBUFFER_EXT);
	switch (status)
	{
	case GL_FRAMEBUFFER_COMPLETE_EXT:
		break;
	case GL_FRAMEBUFFER_UNSUPPORTED_EXT:
		gLog.OutPut("FBO configuration unsupported!\n");
		return false;
//	case GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT:
//		gLog.OutPut("Incomplete multisample buffer. (GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT)(FBO - 820)");
//	return false;

	default:
		gLog.OutPut("FBO programmer error!\n");
		return false;

	}
	gLog.OutPut("OK.\n");
	return true;
}

void BindFBO(FBO fbo, int attachment)
{
	glPushAttrib(GL_VIEWPORT_BIT | GL_COLOR_BUFFER_BIT);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo.fboID);
	glViewport (0, 0, fbo.w, fbo.h);
	glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT+attachment);
}
void UnBindFBO()
{
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	glPopAttrib();
}
void RenderMultipleFBO(unsigned int* buffers, int numBuf)
{
	glDrawBuffers(2, buffers);
}
void CleanupFBO(FBO fbo)
{
	if(fbo.colorID>0)
	{
		for(int n=0;n<fbo.numColors;n++)
		{
			if(fbo.colortexID[n]>0)
				glDeleteTextures(1,&fbo.colortexID[n]);
			glDeleteRenderbuffersEXT(1,&fbo.colorID[n]);
		}
	}
	if(fbo.depthID>0)
	{
		if(fbo.depthtexID>0)
			glDeleteTextures(1,&fbo.depthtexID);
		glDeleteRenderbuffersEXT(1,&fbo.depthID);
	}
	if(fbo.stencilID>0)
	{
		glDeleteRenderbuffersEXT(1,&fbo.stencilID);
	}

	glDeleteFramebuffersEXT(1, &fbo.fboID);
}
