#pragma once
#ifdef WIN32
#include <windows.h>
#endif
#include <gl/glew.h>
#include <vector>
using namespace std;
int CheckGLError(char *file, int line);
#define CHECK_GL_ERROR() CheckGLError(__FILE__, __LINE__)


struct Shader
{
	GLhandleARB	handle;
	unsigned int index;//fast find
};
struct Program
{
	GLhandleARB handle;
	unsigned int index;
};
class ShaderMgr
{
private:
	unsigned int curIndex;

	vector<Shader>	mShaders;
	vector<Program> mPrograms;
	Program*	mCurProgram;
	Shader* FindShader(unsigned int idx);
	Program* FindProgram(unsigned int idx);
	void SetActiveTextureUnit(int unit);
public:
	ShaderMgr(void);
	~ShaderMgr(void);
	unsigned int CreateShader(int type, char* data);//return shader index, or zero if failed
	unsigned int CreateProgram();
	void SetGeometryPrimitive(unsigned int shader, int intype, int outtype, int count);
	void AttachShader(unsigned int shader);//to currently bound program
	void DetachShader(unsigned int shader);//from currently bound program
	void DeleteShader(unsigned int shader);//detaches and delete shader (if can)
	void DeleteProgram(unsigned int prg);
	bool LinkProgram();//link currently bound program
	void BindProgram(unsigned int idx);
	void PassUniform1f(char* var, float value);//on bound program
	void PassUniform2f(char* var, float value1, float value2);//on bound program
	void PassUniform3f(char* var, float value1, float value2, float value3);//on bound program
	void PassUniform4f(char* var, float value1, float value2, float value3, float value4);//on bound program
	void PassUniform1i(char* var, int value);//on bound program
	void PassUniform2i(char* var, int value1, int value2);//on bound program
	void PassUniform3i(char* var, int value1, int value2, int value3);//on bound program
	void PassUniform4i(char* var, int value1, int value2, int value3, int value4);//on bound program
	void PassUniform1fv(char* var, int size, float* value);//on bound program
	void PassUniform2fv(char* var, int size, float* value);//on bound program
	void PassUniform3fv(char* var, int size, float* value);//on bound program
	void PassUniform4fv(char* var, int size, float* value);//on bound program
	void PassUniform1iv(char* var, int size, int* value);//on bound program
	void PassUniform2iv(char* var, int size, int* value);//on bound program
	void PassUniform3iv(char* var, int size, int* value);//on bound program
	void PassUniform4iv(char* var, int size, int* value);//on bound program
	void PassUniformMatrix4f(char* var, int count, bool transpose, float* value);//on bound program

	void PassAttrib1f(int loc, float value);//on bound program
	void PassAttrib2f(int loc, float value1, float value2);//on bound program
	void PassAttrib3f(int loc, float value1, float value2, float value3);//on bound program
	void PassAttrib4f(int loc, float value1, float value2, float value3, float value4);//on bound program
	void PassAttrib1fv(int loc,  float* value);//on bound program
	void PassAttrib2fv(int loc,  float* value);//on bound program
	void PassAttrib3fv(int loc,  float* value);//on bound program
	void PassAttrib4fv(int loc,  float* value);//on bound program
	void PassAttrib4iv(int loc,  int* value);//on bound program

	void PassSampler1D(char* var, int unit, unsigned int texID);
	void PassSampler2D(char* var, int unit, unsigned int texID);
	void PassSampler3D(char* var, int unit, unsigned int texID);
	void PassSamplerRect(char* var, int unit, unsigned int texID);
	void PassSamplerCube(char* var, int unit, unsigned int texID);
	GLint GetUniformLocation(char* var);
	GLint GetAttribLocation(char* var);
};
