#include "Sound.h"
#include "Log.h"


bool InitializeAudio(int numChannels)
{
	gLog.OutPut("[Initializing Audio...]\n");
	// start SDL with audio support
	if(SDL_Init(SDL_INIT_AUDIO)==-1)
	{
		gLog.OutPut("Failed to Initialize audio!\n");
		char output[256];
		sprintf(output, "SDL_Init: %s\n", SDL_GetError());
		gLog.OutPut(output);
		return false;
	}
	// open 44.1KHz, signed 16bit, system byte order,
	//      stereo audio, using 1024 byte chunks
	if(Mix_OpenAudio(44100, MIX_DEFAULT_FORMAT, 2, 1024)==-1)
	{
		gLog.OutPut("Failed to Initialize audio!\n");
		char output[256];
		sprintf(output, "Mix_OpenAudio: %s\n", Mix_GetError());
		gLog.OutPut(output);
		return false;
	}

	int allocated = Mix_AllocateChannels(numChannels);
	char szAllocated[256];
	sprintf(szAllocated,"Number of channels allocated: %i\n",allocated);
	gLog.OutPut(szAllocated);
	gLog.OutPut("Complete.\n");
	return true;
}
void ShutDownAudio()
{
	gLog.OutPut("Shutting down Audio...\n");
	Mix_CloseAudio();
	gLog.OutPut("Done.\n");
}
void PauseAudio()
{
	PauseMasterChannel();
	MasterPauseMusic();
}
void UnPauseAudio()
{
	UnPauseMasterChannel();
	MasterUnPauseMusic();
}
void FadeOutAudio(int ticks)
{
	FadeOutMasterChannel(ticks);
	MasterMusicFadeout(ticks);
}
void MasterVolume(int vol)
{
	Mix_VolumeMusic(vol);
	SetMasterChannelVolume(vol);
}

/***************************************************/
/* Music										   */
/***************************************************/
bool LoadMusic(char* name, Music& music)
{
	
	gLog.OutPut("Loading Music: ");
	gLog.OutPut(name);
	gLog.OutPut("\n");
	// load the MP3 file "music.mp3" to play as music
	music.pMusic = NULL;
	music.pMusic=Mix_LoadMUS(name);
	if(!music.pMusic)
	{
		gLog.OutPut("Failed to Load music!\n");
		char output[256];
		sprintf(output, "Mix_LoadMUS(%s): %s\n",name, Mix_GetError());
		gLog.OutPut(output);
		return false;
	}
	gLog.OutPut("Done\n");
	return true;
}
void FreeMusic(Music& music)
{
	if(music.pMusic)
	{
		music.fPlayedTime = 0;
		Mix_HaltMusic();
		Mix_FreeMusic(music.pMusic);
	}
	music.pMusic = NULL;
}
void PlayMusic(Music music, bool loop, bool pos)
{
	if(music.pMusic)
	{
		if(Mix_PlayMusic(music.pMusic, loop?-1:1)==-1)
		{
			char output[256];
			sprintf(output, "Mix_PlayMusic: %s\n", Mix_GetError());
			gLog.OutPut("Failed to play music reason:\n");
			gLog.OutPut(output);
		}
		if(pos)
		{
			//Mix_RewindMusic();
			if(Mix_SetMusicPosition((double)music.fPlayedTime)==-1)
			{
				char output[256];
				sprintf(output, "Mix_SetMusicPosition: %s\n", Mix_GetError());
				gLog.OutPut(output);

			}
		}
	}
}
void PlayMusicFadeIn(Music music, bool loop, int fadeticks, bool pos)
{
	if(music.pMusic)
	{
		if(Mix_FadeInMusic(music.pMusic, loop?-1:1, fadeticks)==-1)
		{
			char output[256];
			sprintf(output, "Mix_FadeInMusic: %s\n", Mix_GetError());
			gLog.OutPut("Failed to play music reason:\n");
			gLog.OutPut(output);
		}
		if(pos)
		{
//			Mix_RewindMusic();
			if(Mix_SetMusicPosition((double)music.fPlayedTime)==-1)
			{
				char output[256];
				sprintf(output, "Mix_SetMusicPosition: %s\n", Mix_GetError());
				gLog.OutPut(output);

			}

		}
	}
}
void MasterMusicFadeout(int ticks)
{
	Mix_FadeOutMusic(ticks);
}
void MasterPauseMusic()
{
	Mix_PauseMusic();
}
void MasterUnPauseMusic()
{
	Mix_ResumeMusic();
}
void MaserMusicVolume(int vol)
{
	Mix_VolumeMusic(vol);
}
void StopMusic()
{
	Mix_HaltMusic();
}
bool MusicIsPlaying()
{
	return Mix_PlayingMusic()==1?true:false;
}
/***************************************************/
/* Samples										   */
/***************************************************/
bool LoadSample(char* name, Sample& sample)
{
	gLog.OutPut("Loading sample: ");
	gLog.OutPut(name);
	gLog.OutPut("\n");
	sample.pSample = NULL;
	sample.pSample=Mix_LoadWAV(name);
	if(!sample.pSample)
	{
		char output[256];
		sprintf(output, "Mix_LoadWAV: %s\n", Mix_GetError());
		gLog.OutPut("Failed to load sample, reason:\n");
		gLog.OutPut(output);
		return false;
	}
	return true;
}
void FreeSample(Sample& sample)
{
	Mix_FreeChunk(sample.pSample);
	sample.pSample = NULL;
}
void SetChannelVolume(int chan, int vol)//vol 0-128
{
	Mix_Volume(chan,vol);
}
void SetMasterChannelVolume(int vol)
{
	Mix_Volume(-1,vol);
}
void PauseMasterChannel()
{
	Mix_Pause(-1);
}
void UnPauseMasterChannel()
{
	Mix_Resume(-1);
}
void PlaySample(int chan, Sample sample, bool loop)//specify channel
{
	if(sample.pSample)
	if(Mix_PlayChannel(chan, sample.pSample, loop?-1:0)==-1)
	{
		char output[256];
		sprintf(output,"Mix_PlayChannel: %s\n",Mix_GetError());
		gLog.OutPut("Failed to play sample, reason:\n");
		gLog.OutPut(output);
	}
}
int PlaySample(Sample sample, bool loop)//next available channel
{
	int channel = -1;
	if(sample.pSample)
	if((channel=Mix_PlayChannel(-1, sample.pSample, loop?-1:0))==-1)
	{
		char output[256];
		sprintf(output,"Mix_PlayChannel: %s\n",Mix_GetError());
		gLog.OutPut("Failed to play sample, reason:\n");
		gLog.OutPut(output);
	}
	return channel;
}
void StopChannel(int channel)
{
	Mix_HaltChannel(channel);
}
void StopMasterChannel()
{
	Mix_HaltChannel(-1);
}
void FadeInMasterChannel(int ticks)
{
}
void FadeOutMasterChannel(int ticks)
{
	Mix_FadeOutChannel(-1, ticks);
}
void SetSoundPosition(int channel, int angle, int distance)
{
	if(distance>255)
	{
		distance = 255;
	}
	if(!Mix_SetPosition(channel, angle, distance))
	{
		char output[256];
		sprintf(output,"Mix_SetPosition: %s\n", Mix_GetError());
		gLog.OutPut(output);
	}
}
void ReserveChannels(int num)
{
	Mix_ReserveChannels(num);
}
