#include "UTIL_openGL.h"
unsigned int textureMem=0;
GLfloat gMaxAnisotropy = 0;
GLfloat gAnisotropy = 0;

namespace UTIL_GL
{
	void GetGLError()
	{
		GLenum errCode;
		const GLubyte *errString;

		if ((errCode = glGetError()) != GL_NO_ERROR)
		{
			errString = gluErrorString(errCode);
			fprintf (stderr, "OpenGL Error: %s\n", errString);
		}
	}
//================================================================================================//
									/***************
									** 2D helpers **
									****************/
//================================================================================================//
	namespace GL2D
	{
		void SetOrtho(float w, float h)
		{
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			glFrustum(0,w,h,0,1,3);
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
			glTranslatef(0,0,-2);
			glScalef(2,2,2);
		}
		void DrawQuad(float sx, float sy, float ex, float ey)
		{
			glBegin(GL_QUADS);
			glVertex2f(sx,sy);
			glVertex2f(ex,sy);
			glVertex2f(ex,ey);
			glVertex2f(sx,ey);
			glEnd();
		}
		void DrawTextureQuad(float sx, float sy, float ex, float ey)
		{
			glBegin(GL_QUADS);
#ifdef EXTENSION_SUPPORT_1_1 
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0,1);	glVertex2f(sx,sy);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1,1);	glVertex2f(ex,sy);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1,0);	glVertex2f(ex,ey);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0,0);	glVertex2f(sx,ey);
#else
			glTexCoord2f(0,1);	glVertex2f(sx,sy);
			glTexCoord2f(1,1);	glVertex2f(ex,sy);
			glTexCoord2f(1,0);	glVertex2f(ex,ey);
			glTexCoord2f(0,0);	glVertex2f(sx,ey);
#endif
			glEnd();
		}
		void DrawWireBox(float sx, float sy, float ex, float ey)
		{
			glBegin(GL_LINE_LOOP);
			glVertex2f(sx,sy);
			glVertex2f(ex,sy);
			glVertex2f(ex,ey);
			glVertex2f(sx,ey);
			glEnd();
		}
		Vec2 ProjectPointInWindow(Vec3 in, double proj[16], double model[16])
		{
			GLint win[4] = {0,0,640,480};
			double x,y,z;
			Vec2 out;
			gluProject(in.x,in.y,in.z, model, proj, win, &x, &y, &z);
			out.x = (scalar)x;
			out.y = (scalar)y;
			return out;	
		}
	}
//================================================================================================//
									/****************
									 ** 3D helpers **
									 ****************/
//================================================================================================//
	namespace GL3D
	{
		void SetPerspective(float Fov, float Aspect, float zNear, float zFar)
		{
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(Fov, Aspect, zNear, zFar);
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
		}
		void RenderBillBoard(Vec3 pos, Vec3 rgt, Vec3 up, float size)
		{
			Vec3 TL = pos + up * -size + rgt * -size;
			Vec3 TR = pos + up * -size + rgt * size;
			Vec3 DL = pos + up * size + rgt * -size;
			Vec3 DR = pos + up * size + rgt * size;
			glTexCoord2f(0,0); glVertex3fv(TL.v);
			glTexCoord2f(1,0); glVertex3fv(TR.v);
			glTexCoord2f(1,1); glVertex3fv(DR.v);
			glTexCoord2f(0,1); glVertex3fv(DL.v);

/*			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0,0); glVertex3fv(TL.v);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1,0); glVertex3fv(TR.v);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1,1); glVertex3fv(DR.v);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0,1); glVertex3fv(DL.v);
*/
		}
//		Vec3 ProjectPointInWorld(Vec3 in,  double proj[16], double model[16])//make a ray from window, to far plane
		Vec3 ProjectPointInWorld(Vec2 res, Vec2 in)//make a ray from window, to far plane
		{
			GLint win[4] = {0,0,res.x,res.y};
			//glGetIntegerv(GL_VIEWPORT,win);
			double proj[16];
			double model[16];
			glGetDoublev(GL_MODELVIEW_MATRIX,model);
			glGetDoublev(GL_PROJECTION_MATRIX,proj);
			double x,y,z;
			Vec3 out;
			in.y = (float)win[3] - in.y;
			glReadPixels((int)in.x, (int)in.y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &out.z);
			gluUnProject(in.x,in.y,out.z, model, proj, win, &x, &y, &z);
			out.x = (scalar)x;
			out.y = (scalar)y;
			out.z = (scalar)z;
			return out;	
		}
		Vec3 ProjectPointInWorld(Vec2 res, Vec2 in, float& Depth)
		{
			GLint win[4] = {0,0,res.x,res.y};
			//glGetIntegerv(GL_VIEWPORT,win);
			double proj[16];
			double model[16];
			glGetDoublev(GL_MODELVIEW_MATRIX,model);
			glGetDoublev(GL_PROJECTION_MATRIX,proj);
			double x,y,z;
			Vec3 out;
			in.y = (float)win[3] - in.y;
			glReadPixels((int)in.x, (int)in.y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &Depth);
			gluUnProject(in.x,in.y,Depth, model, proj, win, &x, &y, &z);
			out.x = (scalar)x;
			out.y = (scalar)y;
			out.z = (scalar)z;
			return out;	
		}

		Vec3 ProjectRayInWorld(Vec2 res, Vec3 in)//make a ray from window, to far plane
		{
			GLint win[4] = {0,0,res.x,res.y};
			glGetIntegerv(GL_VIEWPORT,win);
			double proj[16];
			double model[16];
			glGetDoublev(GL_MODELVIEW_MATRIX,model);
			glGetDoublev(GL_PROJECTION_MATRIX,proj);
			double x,y,z;
			Vec3 out;
			gluUnProject(in.x,in.y,in.z, model, proj, win, &x, &y, &z);
			out.x = (scalar)x;
			out.y = (scalar)y;
			out.z = (scalar)z;
			return out;	
		}
	}
//================================================================================================//
									/************************
									** texture enviroment **
									************************/
//================================================================================================//
	void TextureWrap(const unsigned int s, const unsigned int t)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, s);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, t);
	}
	void TextureFilter(const unsigned int min, const unsigned int mag)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag);
	}
//================================================================================================//
									/*******************
									** image loading **
									*******************/
//================================================================================================//
	namespace Image
	{
		ILuint ImagePrivate::DevilLoadImage(std::string name)
		{
			ILuint ImageName;
			ilGenImages(1, &ImageName);
			ilBindImage(ImageName);
			if(!ilLoadImage((char*)name.c_str()))
			{
				string str = "Failed to load image: " + name + "!\n";
				gLog.OutPut(str);
				return 0;
			}
			textureMem += (ilGetInteger(IL_IMAGE_WIDTH)*ilGetInteger(IL_IMAGE_HEIGHT)*4);
			string str = "Image Loaded: " + name + "\n";
			gLog.OutPut(str);
	
			return ImageName;
		}
		GLuint LoadImage(std::string name)
		{
			ILuint ImageName;
			if((ImageName=ImagePrivate::DevilLoadImage(name))==0)
				return 0;
			GLuint ID = ilutGLBindTexImage();
			if(gAnisotropy>0)
			{
				UTIL_GL::BindTexture(GL_TEXTURE_2D, ID);
				glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, gAnisotropy);
			}
//			GLuint ID = 1;
			ilDeleteImages(1, &ImageName);
			return ID;		
		}
		GLuint LoadImage1D(std::string name)
		{
			ILuint ImageName;
			if((ImageName=ImagePrivate::DevilLoadImage(name))==0)
				return 0;
			unsigned int  height,bpp;
//			width = ilGetInteger(IL_IMAGE_WIDTH);
			height = ilGetInteger(IL_IMAGE_HEIGHT);
			bpp = ilGetInteger(IL_IMAGE_BPP);
			unsigned char* data=NULL;
			data = new unsigned char [height*bpp];
			memcpy(data,ilGetData(),height*bpp);
			ilDeleteImages(1, &ImageName);
			unsigned int ID=0;
			glEnable(GL_TEXTURE_1D);
			glGenTextures(1,&ID);
			glBindTexture(GL_TEXTURE_1D, ID);
//			TextureFilter(min,mag);
//			TextureWrap(wrapx,wrapy);

			switch(bpp)
			{
			case 3:
				glTexImage1D(GL_TEXTURE_2D,0,GL_RGB,height,0,GL_RGB,GL_UNSIGNED_BYTE,data);
				break;
			case 4:
				glTexImage1D(GL_TEXTURE_2D,0,GL_RGBA,height,0,GL_RGBA,GL_UNSIGNED_BYTE,data);
				break;
			default:
				glTexImage1D(GL_TEXTURE_2D,0,GL_RGB,height,0,GL_RGB,GL_UNSIGNED_BYTE,data);
				break;
			}
			glDisable(GL_TEXTURE_2D);
			delete data;
			return ID;
		}


		GLuint LoadImage(std::string name, unsigned int& width, unsigned int& height)
		{
			ILuint ImageName;
			if((ImageName=ImagePrivate::DevilLoadImage(name))==0)
				return 0;
			//GLuint ID = ilutGLBindTexImage();
			GLuint ID = 0;
			width = ilGetInteger(IL_IMAGE_WIDTH);
			height = ilGetInteger(IL_IMAGE_HEIGHT);
			ilDeleteImages(1, &ImageName);
			if(!(ID=LoadImage(name,GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
				return 0;
			return ID;
		}
		GLuint LoadImage(std::string name, const int min, const int mag, const int wrapx, const int wrapy)
		{
			ILuint ImageName;
			if((ImageName=ImagePrivate::DevilLoadImage(name))==0)
				return 0;
			unsigned int  width,height,bpp;
			width = ilGetInteger(IL_IMAGE_WIDTH);
			height = ilGetInteger(IL_IMAGE_HEIGHT);
			bpp = ilGetInteger(IL_IMAGE_BPP);
			unsigned char* data=NULL;
			data = new unsigned char [width*height*bpp];
			memcpy(data,ilGetData(),width*height*bpp);
			ilDeleteImages(1, &ImageName);
			unsigned int ID=0;
			glEnable(GL_TEXTURE_2D);
			glGenTextures(1,&ID);
			glBindTexture(GL_TEXTURE_2D, ID);
			TextureFilter(min,mag);
			TextureWrap(wrapx,wrapy);
			if(name.find(".tga")!=std::string::npos)
			{
				for(unsigned int n=0; n<width*height*bpp; n+=bpp)
				{
					unsigned char temp = data[n];
					data[n] = data[n+2];
					data[n+2] = temp;
				}
			}
			switch(bpp)
			{
			case 3:
				glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,width,height,0,GL_RGB,GL_UNSIGNED_BYTE,data);
				break;
			case 4:
				glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,width,height,0,GL_RGBA,GL_UNSIGNED_BYTE,data);
				break;
			default:
				glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,width,height,0,GL_RGB,GL_UNSIGNED_BYTE,data);
				break;
			}
			glDisable(GL_TEXTURE_2D);
			delete data;
			return ID;
		}
		GLuint LoadImageMipMapped(std::string name)
		{
			unsigned int w,h,b,ID;
			unsigned char* data=NULL;
	 		if(data = LoadImageData((char*)name.c_str(),w,h,b))
			{
				glGenTextures(1,&ID);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D,ID);
				UTIL_GL::TextureFilter(GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR );
				if(b==4)
					gluBuild2DMipmaps(GL_TEXTURE_2D,b,w,h,GL_RGBA,GL_UNSIGNED_BYTE,data);
				else if(b==3)
					gluBuild2DMipmaps(GL_TEXTURE_2D,b,w,h,GL_RGB,GL_UNSIGNED_BYTE,data);
				delete [] data;
				glDisable(GL_TEXTURE_2D);
				return ID;
			}
			return 0;//*/
		}
		unsigned char*  LoadImageData(std::string name, unsigned int& width, unsigned int& height, unsigned int& bpp)
		{
			ILuint ImageName;
			if((ImageName=ImagePrivate::DevilLoadImage(name))==0)
			{
				string str = "Failed to load image data: " + name + "!\n";
				gLog.OutPut(str);
				return NULL;
			}
			UTIL_GL::TextureFilter(GL_NEAREST,GL_NEAREST);
			width = ilGetInteger(IL_IMAGE_WIDTH);
			height = ilGetInteger(IL_IMAGE_HEIGHT);
			bpp = ilGetInteger(IL_IMAGE_BPP);
			unsigned char* data=NULL;
			data = new unsigned char [width*height*bpp];
			memcpy(data,ilGetData(),width*height*bpp);
			ilDeleteImages(1, &ImageName);
			return data;
		}
	}

//================================================================================================//
									/***************
									 ** shortcuts **
									 ***************/
//================================================================================================//
	void BindTexture(const unsigned int type, unsigned int id)
	{
		glEnable(type);
		glBindTexture(type,id);
	}
	void SetBlend(const unsigned int src, const unsigned int dst)
	{
		glEnable(GL_BLEND);
		glBlendFunc(src,dst);
	}
	void AlphaTest(const unsigned int func, float alpha)
	{
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(func,alpha);
	}
	void DepthTest(const unsigned int func)
	{
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(func);
	}
	void StencilTest(const unsigned int func, int ref, unsigned int mask)
	{
		glEnable(GL_STENCIL_TEST);
		glStencilFunc(func,ref,mask);
	}
#ifdef EXTENSION_SUPPORT_1_1 
	void BindMultiTexture(const unsigned int channel, unsigned int id)
	{
		glActiveTextureARB(channel);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, id);
	}
	void DisableTextureUnit(const unsigned int channel)
	{
		glActiveTextureARB(channel);
		glBindTexture(GL_TEXTURE_2D, 0);
		glDisable(GL_TEXTURE_2D);
	}
#endif
}