#include "VoxelObject.h"
#include "Engine.h"

bool PruneVoxelObjects(VoxelObject* obj)
{
	//obj->miLifeTime--;
	if(obj->mbRemoveMe || obj->miLifeTime<=0)
	{
		delete obj;
		return true;
	}
	return false;
}
VoxelObject::VoxelObject()
{
	mVoxels = NULL;
	mChunk = NULL;
	mRigidBody = NULL;
	mvW= mvH= mvD = 0;
	mbRemoveMe = false;
	miLifeTime = 1000;
}
VoxelObject::~VoxelObject()
{
	if(mChunk)
	{
		if(mChunk->mCompound)
		{
			while(mChunk->mCompound->getNumChildShapes()>0)
			{
				delete mChunk->mCompound->getChildShape(0);
				mChunk->mCompound->removeChildShapeByIndex(0);
			}
		}
		delete mChunk;
	}

	if(mRigidBody)
	{
		gpEng->mBulletDynamicsWorld->removeRigidBody(mRigidBody);
		delete mRigidBody->getMotionState();
		delete mRigidBody;
	}

	if(mVoxels)
		delete [] mVoxels;
}
void VoxelObject::InitializeObject(unsigned char* voxels, unsigned int w, unsigned int h, unsigned int d, Vec3 Angles, Vec3 Pos, Vec3 Origin, float mass)
{
	gLog.OutPut("Init vobject\n");
	mVoxels = new unsigned char[(w*h*(d+1))];
	gLog.OutPut("copy voxels\n");
	//memcpy(mVoxels,voxels,w*h*d);
	for(int n=0; n<w*h*d; n++)
		mVoxels[n] = voxels[n];
	mvW = w;
	mvH = h;
	mvD = d;
	mOrigin = Origin;
	mfMass = mass;
	mfMass = w*d*h;
	mfBoundRadius = max(mvH,max(mvW,mvD)) / 2;
	gLog.OutPut("generate chunk\n");
	mChunk = GenerateChunk(0,0,0,w,d,h,Origin);
	mChunk->mCompound = new btCompoundShape(true);
	gLog.OutPut("generate shapes\n");
	int VoxCount = 0;
	for(int n=0; n<(int)mChunk->Faces.size(); n++)
	{
		vFace* FACE = &mChunk->Faces[n]; 
//		if(FACE->E.x<2 && FACE->E.y<2 && FACE->E.z<2)
//			continue;
		VoxCount += (FACE->E.x*2)*(FACE->E.y*2)*(FACE->E.z*2);
		btCollisionShape* boxShape = new btBoxShape(btVector3(FACE->E.x,FACE->E.y,FACE->E.z));
		btTransform trans;
		trans.setIdentity();
		trans.setOrigin(btVector3(FACE->V.x,FACE->V.y,FACE->V.z));
		mChunk->mCompound->addChildShape(trans, boxShape);

	}
	if(VoxCount < 15)
	{
		mbRemoveMe = true;
		gLog.OutPut("body rejected. too small\n");
	}
	gLog.OutPut("generate rigidbody\n");

	btVector3 btinertia;
	mChunk->mCompound->calculateLocalInertia(mfMass,btinertia);
	Vec3 inertia(btinertia.x(),btinertia.y(),btinertia.z());

	btDefaultMotionState* MotionState = new btDefaultMotionState(btTransform(btQuaternion(DEG2RAD(Angles.x),DEG2RAD(Angles.y),DEG2RAD(Angles.z)),btVector3(Pos.x,Pos.y,Pos.z)));
	btRigidBody::btRigidBodyConstructionInfo RigidBodyCI(mfMass,MotionState,mChunk->mCompound,btVector3(inertia.x,inertia.y,inertia.z));
	mRigidBody = new btRigidBody(RigidBodyCI);
	gpEng->mBulletDynamicsWorld->addRigidBody(mRigidBody);
//	mBulletRigidBodies.push_back(mRigidBody);
	gLog.OutPut("done\n");
}

vChunk* VoxelObject::GenerateChunk(int x, int y, int z, int w, int d, int h, Vec3 Origin)
{
	vChunk* Chunk = new vChunk;
	Chunk->mCompound = NULL;
	Chunk->Faces.clear();
	//create temp around to flag if the voxel has been visited
	unsigned char* Status = new unsigned char[(w*h*(d+1))+1];
	memset(Status,0,(w*h*(d+1))+1);

	int voxelCount=0;
	int x2, y2, z2;
	int px, py, pz;
	int spanx, spany, spanz;
	for(px=x;px<x+w;px++)
	for(py=y;py<y+d;py++)
	for(pz=z;pz<z+h;pz++)
	{
		if(Status[((py-y)*h+(pz-z))*w+(px-x)]>0)
			continue;
		unsigned char Palette = mVoxels[(py*h+pz)*w+px];
		if(Palette==0)
			continue;
		vFace VC;
		voxelCount++;
//	gLog.OutPut("span\n");
		spanx = (w)-(px-x);
		spany = (d)-(py-y);
		spanz = (h)-(pz-z);
		y2=0;
		while(y2<spany)
		{
			z2=0;
			while(z2<spanz)
			{
				x2=1;
				while(x2<spanx)
				{
					if(mVoxels[((py+y2)*h+(pz+z2))*w+(px+x2)]!=Palette || Status[((py+y2-y)*h+(pz+z2-z))*w+(px+x2-x)]>0)
						break;
					x2++;
				}
				spanx = x2;
				z2++;
				if(mVoxels[((py+y2)*h+(pz+z2))*w+(px)]!=Palette || Status[((py+y2-y)*h+(pz+z2-z))*w+(px-x)]>0)
					break;
			}
			spanz = z2;
			y2++;
			if(mVoxels[((py+y2)*h+(pz))*w+(px)]!=Palette || Status[((py+y2-y)*h+(pz-z))*w+(px-x)]>0)
				break;
		}
		spany = y2;
//		if(spanx ==0 || spany ==0 || spanz ==0)
//			gLog.OutPut("problem\n");
		bool Encased = true;

//		gLog.OutPut("encase\n");
		if(px==0||py==0||pz==0 || px+spanx==x+w-1 || py+spany==y+d-1 || pz+spanz==z+h-1)
			Encased =false;
		else
		for(x2=0; x2<spanx && x2<w-px; x2++)
		for(y2=0; y2<spany && y2<d-py; y2++)
		{
			if(pz>0 && mVoxels[((py+y2)*h+(pz-1))*w+(px+x2)] ==0 )
				Encased = false;
			if(pz+spanz+1 < h && mVoxels[((py+y2)*h+(pz+spanz+1))*w+(px+x2)] ==0)
				Encased = false;

			if(!Encased)
				break;
			for(z2=0; z2<spanz && z2<h-pz; z2++)
			{
				if(px>0 && mVoxels[((py+y2)*h+(pz+z2))*w+(px-1)] ==0 )
					Encased = false;
				if(px+spanx+1 < w && mVoxels[((py+y2)*h+(pz+z2))*w+(px+spanx+1)] ==0)
					Encased = false;

				if(py>0 && mVoxels[((py-1)*h+(pz+z2))*w+(px+x2)] ==0 )
					Encased = false;
				if(py+spany+1 < d && mVoxels[((py+spany+1)*h+(pz+z2))*w+(px+x2)] ==0)
					Encased = false;

				if(!Encased)
					break;
			}
		}
//		gLog.OutPut("tri\n");
		if(!Encased)
		{
			VC.E.x = (float)(spanx)/2;
			VC.E.y = (float)(spany)/2;
			VC.E.z = (float)(spanz)/2;
			VC.V.x = (px + VC.E.x) - Origin.x;
			VC.V.y = (py + VC.E.y) - Origin.y;
			VC.V.z = (pz + VC.E.z) - Origin.z;
			VC.Tex1[0] = mVoxels[(py*h+pz)*w+px];//UTIL_Misc::Rand() % 256;
			VC.Tex1[1] = 0;
			Chunk->Faces.push_back(VC);
		}
//		else
//			gLog.OutPut("Volume discarded\n");
//		gLog.OutPut("status\n");
		for(x2=0;x2<spanx;x2++)
		for(y2=0;y2<spany;y2++)
		for(z2=0;z2<spanz;z2++)
			Status[((py+y2-y)*h+(pz+z2-z))*w+(px+x2-x)] = 1;
		//memset(Status+((py+y2-y)*h+(pz+z2-z))*w+(px-x),1,spanx);
		pz+=spanz-1;
		//gLog.OutPut("added volume!\n");
	}

	if(voxelCount==0)
		mbRemoveMe = true;
	delete [] Status;
	return Chunk;
}
void VoxelObject::Update()
{
}
void VoxelObject::Render(const float interp, int PASS)
{
	if(mRigidBody==NULL)
		return;
	if(mChunk==NULL)
		return;
	if(mChunk->Faces.empty())
		return;

	glPushMatrix();
	btTransform trans;
	mRigidBody->getMotionState()->getWorldTransform(trans);
	float MATRIX[16];
	trans.getOpenGLMatrix(MATRIX);

	if(PASS == 1)
	{
		//glMultMatrixf(MATRIX);
		gpEng->mShaderMgr.PassUniformMatrix4f("model",1,false,MATRIX);
	 	glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glVertexPointer(3, GL_FLOAT, sizeof(vFace), &mChunk->Faces[0].V.x);
		glNormalPointer(GL_FLOAT, sizeof(vFace), &mChunk->Faces[0].E.x);
		glTexCoordPointer(2, GL_SHORT, sizeof(vFace), &mChunk->Faces[0].Tex1[0]);
		glDrawArrays(GL_POINTS,0,mChunk->Faces.size());
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_NORMAL_ARRAY);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	}
	else if(PASS == 0)//depth only pass
	{
		glMultMatrixf(MATRIX);
	 	glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);
		glVertexPointer(3, GL_FLOAT, sizeof(vFace), &mChunk->Faces[0].V.x);
		glNormalPointer(GL_FLOAT, sizeof(vFace), &mChunk->Faces[0].E.x);
		glDrawArrays(GL_POINTS,0,mChunk->Faces.size());
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_NORMAL_ARRAY);
	}
	glPopMatrix();
}

void VoxelObject::SubtractSphere(Vec3 P, float R)
{
	btTransform trans;
	mRigidBody->getMotionState()->getWorldTransform(trans);
	btVector3 btP = trans.getOrigin();
	if(Distance(P,Vec3(btP.x(), btP.y(), btP.z())) > R+mfBoundRadius)
		return;
//	gLog.OutPut("subtract sphere\n");
	char sz[256];
	float MATRIX[16];
	trans.getOpenGLMatrix(MATRIX);
	btQuaternion Q = trans.getRotation();
	
	Vec3 POS = Vec3(btP.x(), btP.y(), btP.z());
	P = P-POS;
	btVector3 A = Q.getAxis();
	Vec3 NP = ArbitraryRotate(P,-Q.getAngle(),Vec3(A.x(),A.y(),A.z()));
	float HR = R/2;


	NP += Vec3(mvW/2,mvD/2,mvH/2);
	//do the subtraction
	for(int x=NP.x-R; x<NP.x+R; x++)
	{
		if(x<0 || x>=mvW) continue;
		for(int y=NP.y-R; y<NP.y+R; y++)
		{
			if(y<0 || y>=mvD) continue;
			for(int z=NP.z-R; z<NP.z+R; z++)
			{
				if(z<0 || z>=mvH) continue;
				if(Distance(Vec3(x,y,z),NP)<=R)
				{
					mVoxels[(y*mvH+z)*mvW+x] = 0;
/*					mVoxels[(y*mvH+z)*mvW+(x+1)] = 0;
					mVoxels[(y*mvH+(z+1))*mvW+x] = 0;
					mVoxels[(y*mvH+(z+1))*mvW+(x+1)] = 0;
					mVoxels[((y+1)*mvH+z)*mvW+x] = 0;
					mVoxels[((y+1)*mvH+z)*mvW+(x+1)] = 0;
					mVoxels[((y+1)*mvH+(z+1))*mvW+x] = 0;
					mVoxels[((y+1)*mvH+(z+1))*mvW+(x+1)] = 0;*/
				}

			}
		}
	}
	
	//floodfill to find if broken
	StartFloodFill(NP-Vec3(R+2,R+2,R+2), NP+Vec3(R+2,R+2,R+2));

//	gLog.OutPut("cleanup\n");
	//cleanup existing
	if(mChunk)
	{
		if(mChunk->mCompound)
		{
			while(mChunk->mCompound->getNumChildShapes()>0)
			{
				delete mChunk->mCompound->getChildShape(0);
				mChunk->mCompound->removeChildShapeByIndex(0);
			}
		}
		delete mChunk;
	}

	btVector3 btVel,btAVel;
	if(mRigidBody)
	{
		btVel = mRigidBody->getLinearVelocity();
		btAVel = mRigidBody->getAngularVelocity();
		gpEng->mBulletDynamicsWorld->removeRigidBody(mRigidBody);
		delete mRigidBody->getMotionState();
		delete mRigidBody;
	}


//	gLog.OutPut("chunk\n");
	mChunk = GenerateChunk(0,0,0,mvW,mvD,mvH,mOrigin);
//	gLog.OutPut("shape\n");
	mChunk->mCompound = new btCompoundShape(true);
	for(int n=0; n<(int)mChunk->Faces.size(); n++)
	{
		vFace* FACE = &mChunk->Faces[n]; 
		btCollisionShape* boxShape = new btBoxShape(btVector3(FACE->E.x,FACE->E.y,FACE->E.z));
		btTransform trans;
		trans.setIdentity();
		trans.setOrigin(btVector3(FACE->V.x,FACE->V.y,FACE->V.z));
		mChunk->mCompound->addChildShape(trans, boxShape);
	}

//	gLog.OutPut("body\n");
	btVector3 btinertia;
	mChunk->mCompound->calculateLocalInertia(mfMass,btinertia);
	Vec3 inertia(btinertia.x(),btinertia.y(),btinertia.z());

	btDefaultMotionState* MotionState = new btDefaultMotionState(btTransform(Q,btP));
	btRigidBody::btRigidBodyConstructionInfo RigidBodyCI(mfMass,MotionState,mChunk->mCompound,btVector3(inertia.x,inertia.y,inertia.z));
	mRigidBody = new btRigidBody(RigidBodyCI);
	mRigidBody->setAngularVelocity(btAVel);
	mRigidBody->setLinearVelocity(btVel);
	gpEng->mBulletDynamicsWorld->addRigidBody(mRigidBody);

}

void VoxelObject::StartFloodFill(Vec3 min, Vec3 max)
{
	unsigned char* mask = new unsigned char[(mvW*mvH*(mvD+1))+1];
	memset(mask,0,(mvW*mvH*(mvD+1))+1);

	char sz[256];
	Vec3 outmin,outmax;
	if(min.x<0)	min.x=0;
	if(min.y<0) min.y=0;
	if(min.z<0) min.z=0;
	if(max.x>mvW) max.x=mvW;
	if(max.y>mvD) max.y=mvD;
	if(max.z>mvH) max.z=mvH;
	unsigned char newcol = 1;

	for(int x = min.x; x<max.x; x++)
	{
		for(int y = min.y; y<max.y; y++)
		{
			for(int z = min.z; z<max.z; z++)
			{
				newcol++;
				outmin = outmax = Vec3(x,y,z);
				int voxCount;
				if(PointFill(mask,x,y,z,outmin,outmax,newcol,voxCount))
				{
					mbRemoveMe = true;
					if(outmin == outmax)
						continue;
					Vec3 minBox = Vec3(0,0,0), maxBox = Vec3(mvW,mvD,mvH);
					if(outmin.x>minBox.x) minBox.x = outmin.x;
					if(outmin.y>minBox.y) minBox.y = outmin.y;
					if(outmin.z>minBox.z) minBox.z = outmin.z;
					if(outmax.x<maxBox.x) maxBox.x = outmax.x;
					if(outmax.y<maxBox.y) maxBox.y = outmax.y;
					if(outmax.z<maxBox.z) maxBox.z = outmax.z;

					sprintf(sz,"new hull: MIN %i %i %i MAX %i %i %i\n",  (int)minBox.x, (int)minBox.y, (int)minBox.z, (int)maxBox.x, (int)maxBox.y, (int)maxBox.z);
					gLog.OutPut(sz);
					if( (int)minBox.x == (int)maxBox.x || 
						(int)minBox.y == (int)maxBox.y || 
						(int)minBox.z == (int)maxBox.z )
					{
						gLog.OutPut("object removed\n");
						continue;
					}
					maxBox += Vec3(1,1,1);
					int sx = minBox.x;
					int sy = minBox.y;
					int sz = minBox.z;
					int ex = maxBox.x-sx;
					int ey = maxBox.y-sy;
					int ez = maxBox.z-sz;

					gLog.OutPut("new voxel object created!\n");

					unsigned char* NewVox = new unsigned char[ex*ez*ey];
					memset(NewVox,0,ex*ez*ey);
					for(int tx=sx; tx<sx+ex; tx++)
					for(int ty=sy; ty<sy+ey; ty++)
					for(int tz=sz; tz<sz+ez; tz++)
					{
						if(mask[(ty*mvH+tz)*mvW+tx] == newcol)
						NewVox[((ty-sy)*ez+(tz-sz))*ex+(tx-sx)] = mVoxels[(ty*mvH+tz)*mvW+tx];
					}
					VoxelObject* VB = new VoxelObject();
					VB->InitializeObjectFromBody(NewVox,ex,ez,ey,
						mRigidBody,
						Vec3(ex/2,ey/2,ez/2),
						mfMass,
						Vec3(mvW/2,mvD/2,mvH/2) - (minBox+Vec3(ex/2,ey/2,ez/2)) );


					gpEng->mVoxelObjects.push_front(VB);
					delete [] NewVox;
				}
			}
		}
	}
	delete [] mask;
}

bool VoxelObject::PointFill(unsigned char* mask, int x, int y, int z, Vec3& min, Vec3& max, unsigned char newcol, int& outcount)
{
/*	if(mask[(y*mvH+z)*mvW+x]>0)
		return false;

	if(mVoxels[(y*mvH+z)*mvW+x]==0)
		return false;
	mask[(y*mvH+z)*mvW+x] = newcol;

	if(x<min.x) min.x = x;
	if(y<min.y) min.y = y;
	if(z<min.z) min.z = z;
	if(x>max.x) max.x = x;
	if(y>max.y) max.y = y;
	if(z>max.z) max.z = z;

	if(x>0)
		PointFill(mask,x-1,y,z,min,max,newcol);
	if(x<mvW-1)
		PointFill(mask,x+1,y,z,min,max,newcol);
	if(y>0)
		PointFill(mask,x,y-1,z,min,max,newcol);
	if(y<mvD-1)
		PointFill(mask,x,y+1,z,min,max,newcol);
	if(z>0)
		PointFill(mask,x,y,z-1,min,max,newcol);
	if(z<mvH-1)
		PointFill(mask,x,y,z+1,min,max,newcol);

	return true;
	*/

	if(mask[(y*mvH+z)*mvW+x]>0 || mVoxels[(y*mvH+z)*mvW+x]==0)
		return 0;

	outcount=0;
	int count = 100000;
	int idx = 0;
	std::vector<Vec3> stack;
	stack.push_back(Vec3(x,y,z));
	Vec3 V;
	while(count>0)
	{
		if(stack.empty())
			break;
		V = stack.back();
		stack.pop_back();
		idx = (V.y*mvH+V.z)*mvW+V.x;
		if(mask[idx]>0) 
			continue;
		mask[idx] = newcol;
		if(mVoxels[idx]==0)
			continue;
		outcount++;
		if(V.x<min.x) min.x = V.x;
		if(V.y<min.y) min.y = V.y;
		if(V.z<min.z) min.z = V.z;
		if(V.x>max.x) max.x = V.x;
		if(V.y>max.y) max.y = V.y;
		if(V.z>max.z) max.z = V.z;

		if(V.x>0)
			stack.push_back(Vec3(V.x-1,V.y,V.z));
		if(V.x<mvW-1)
			stack.push_back(Vec3(V.x+1,V.y,V.z));
		if(V.y>0)
			stack.push_back(Vec3(V.x,V.y-1,V.z));
		if(V.y<mvD-1)
			stack.push_back(Vec3(V.x,V.y+1,V.z));
		if(V.z>0)
			stack.push_back(Vec3(V.x,V.y,V.z-1));
		if(V.z<mvH-1)
			stack.push_back(Vec3(V.x,V.y,V.z+1));
		count--;
	}

	printf("fill finished with a count of %i\n",count);
	if(count>0)
		return true;
	return false;

}

void VoxelObject::InitializeObjectFromBody(unsigned char* voxels, unsigned int w, unsigned int h, unsigned int d, btRigidBody* body, Vec3 Origin, float mass, Vec3 Center)
{
	gLog.OutPut("Init child vobject\n");
	mVoxels = new unsigned char[(w*h*(d+1))];
	gLog.OutPut("copy voxels\n");
	for(int n=0; n<w*h*d; n++)
		mVoxels[n] = voxels[n];
	mvW = w;
	mvH = h;
	mvD = d;
	mOrigin = Origin;
	mfMass = mass;
	mfMass = w*d*h;
	mfBoundRadius = max(mvH,max(mvW,mvD)) / 2;
	gLog.OutPut("generate chunk\n");
	mChunk = GenerateChunk(0,0,0,w,d,h,Origin);
	mChunk->mCompound = new btCompoundShape(true);
	gLog.OutPut("generate shapes\n");
	int VoxCount = 0;
	for(int n=0; n<(int)mChunk->Faces.size(); n++)
	{
		vFace* FACE = &mChunk->Faces[n]; 
//		if(FACE->E.x<2 && FACE->E.y<2 && FACE->E.z<2)
//			continue;
		VoxCount += (FACE->E.x*2)*(FACE->E.y*2)*(FACE->E.z*2);
		btCollisionShape* boxShape = new btBoxShape(btVector3(FACE->E.x,FACE->E.y,FACE->E.z));
		btTransform trans;
		trans.setIdentity();
		trans.setOrigin(btVector3(FACE->V.x,FACE->V.y,FACE->V.z));
		mChunk->mCompound->addChildShape(trans, boxShape);
	}
	if(VoxCount < 15)
	{
		mbRemoveMe = true;
		gLog.OutPut("body rejected. too small\n");
	}
	gLog.OutPut("generate rigidbody\n");

	btVector3 btinertia;
	mChunk->mCompound->calculateLocalInertia(mfMass,btinertia);
	Vec3 inertia(btinertia.x(),btinertia.y(),btinertia.z());


	btTransform bodytrans;
	body->getMotionState()->getWorldTransform(bodytrans);
	btQuaternion Q = bodytrans.getRotation();
	btVector3 btP = bodytrans.getOrigin();
	Vec3 P = Vec3(btP.x(),btP.y(),btP.z());
	btVector3 btAxis = Q.getAxis();
	P -= ArbitraryRotate(Center,Q.getAngle(),Vec3(btAxis.x(),btAxis.y(),btAxis.z()));
	btP = btVector3(P.x,P.y,P.z);
	btDefaultMotionState* MotionState = new btDefaultMotionState(btTransform(Q,btP));
	btRigidBody::btRigidBodyConstructionInfo RigidBodyCI(mfMass,MotionState,mChunk->mCompound,btVector3(inertia.x,inertia.y,inertia.z));
	mRigidBody = new btRigidBody(RigidBodyCI);
	gpEng->mBulletDynamicsWorld->addRigidBody(mRigidBody);
	mRigidBody->setAngularVelocity(body->getAngularVelocity());
	mRigidBody->setLinearVelocity(body->getLinearVelocity());
	gLog.OutPut("done\n");
	
}